#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2021 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""The MODIS Reader operates on HDF4 Level 1B files from the Moderate Resolution
Imaging Spectroradiometer (MODIS) instruments on the Aqua and Terra
satellites. The reader is designed to work with files created by the IMAPP
direct broadcast processing system (file naming conventions such as
a1.17006.1855.1000m.hdf), but can support other types of L1B files, including
the NASA archived files (file naming conventions such as
MOD021KM.A2017004.1732.005.2017023210017.hdf).  The
reader can be specified to the ``polar2grid.sh`` script by using the reader
name ``modis`` or ``modis_l1b``.

This reader’s default remapping algorithm is ewa for Elliptical Weighted
Averaging resampling. The ``--weight-delta-max`` parameter set to 10
and the ``--weight-distance-max`` parameter set to 1.

It provides the following products:

    +--------------------+-----------------------------------------------------+
    | Product Name       | Description                                         |
    +====================+=====================================================+
    | vis01              | Visible 1 Band                                      |
    +--------------------+-----------------------------------------------------+
    | vis02              | Visible 2 Band                                      |
    +--------------------+-----------------------------------------------------+
    | vis03              | Visible 3 Band                                      |
    +--------------------+-----------------------------------------------------+
    | vis04              | Visible 4 Band                                      |
    +--------------------+-----------------------------------------------------+
    | vis05              | Visible 5 Band                                      |
    +--------------------+-----------------------------------------------------+
    | vis06              | Visible 6 Band                                      |
    +--------------------+-----------------------------------------------------+
    | vis07              | Visible 7 Band                                      |
    +--------------------+-----------------------------------------------------+
    | vis26              | Visible 26 Band                                     |
    +--------------------+-----------------------------------------------------+
    | bt20               | Brightness Temperature Band 20                      |
    +--------------------+-----------------------------------------------------+
    | bt21               | Brightness Temperature Band 21                      |
    +--------------------+-----------------------------------------------------+
    | bt22               | Brightness Temperature Band 22                      |
    +--------------------+-----------------------------------------------------+
    | bt23               | Brightness Temperature Band 23                      |
    +--------------------+-----------------------------------------------------+
    | bt24               | Brightness Temperature Band 24                      |
    +--------------------+-----------------------------------------------------+
    | bt25               | Brightness Temperature Band 25                      |
    +--------------------+-----------------------------------------------------+
    | bt27               | Brightness Temperature Band 27                      |
    +--------------------+-----------------------------------------------------+
    | bt28               | Brightness Temperature Band 28                      |
    +--------------------+-----------------------------------------------------+
    | bt29               | Brightness Temperature Band 29                      |
    +--------------------+-----------------------------------------------------+
    | bt30               | Brightness Temperature Band 30                      |
    +--------------------+-----------------------------------------------------+
    | bt31               | Brightness Temperature Band 31                      |
    +--------------------+-----------------------------------------------------+
    | bt32               | Brightness Temperature Band 32                      |
    +--------------------+-----------------------------------------------------+
    | bt33               | Brightness Temperature Band 33                      |
    +--------------------+-----------------------------------------------------+
    | bt34               | Brightness Temperature Band 34                      |
    +--------------------+-----------------------------------------------------+
    | bt35               | Brightness Temperature Band 35                      |
    +--------------------+-----------------------------------------------------+
    | bt36               | Brightness Temperature Band 36                      |
    +--------------------+-----------------------------------------------------+
    | ir20               | Radiance Band 20                                    |
    +--------------------+-----------------------------------------------------+
    | ir21               | Radiance Band 21                                    |
    +--------------------+-----------------------------------------------------+
    | ir22               | Radiance Band 22                                    |
    +--------------------+-----------------------------------------------------+
    | ir23               | Radiance Band 23                                    |
    +--------------------+-----------------------------------------------------+
    | ir24               | Radiance Band 24                                    |
    +--------------------+-----------------------------------------------------+
    | ir25               | Radiance Band 25                                    |
    +--------------------+-----------------------------------------------------+
    | ir27               | Radiance Band 27                                    |
    +--------------------+-----------------------------------------------------+
    | ir28               | Radiance Band 28                                    |
    +--------------------+-----------------------------------------------------+
    | ir29               | Radiance Band 29                                    |
    +--------------------+-----------------------------------------------------+
    | ir30               | Radiance Band 30                                    |
    +--------------------+-----------------------------------------------------+
    | ir31               | Radiance Band 31                                    |
    +--------------------+-----------------------------------------------------+
    | ir32               | Radiance Band 32                                    |
    +--------------------+-----------------------------------------------------+
    | ir33               | Radiance Band 33                                    |
    +--------------------+-----------------------------------------------------+
    | ir34               | Radiance Band 34                                    |
    +--------------------+-----------------------------------------------------+
    | ir35               | Radiance Band 35                                    |
    +--------------------+-----------------------------------------------------+
    | ir36               | Radiance Band 36                                    |
    +--------------------+-----------------------------------------------------+
    | fog                | Temperature Difference between BT31                 |
    |                    | and BT20                                            |
    +--------------------+-----------------------------------------------------+
    | true_color         | Ratio sharpened rayleigh corrected true color       |
    +--------------------+-----------------------------------------------------+
    | false_color        | Ratio sharpened rayleigh corrected false color      |
    +--------------------+-----------------------------------------------------+

For reflectance/visible products a check is done to make sure that at least
10% of the swath is day time. Data is considered day time where solar zenith
angle is less than 90 degrees.

"""

from __future__ import annotations

import argparse
from argparse import ArgumentParser, _ArgumentGroup
from typing import Optional

from satpy import DataQuery

from polar2grid.core.script_utils import ExtendConstAction

from ._base import ReaderProxyBase

PREFERRED_CHUNK_SIZE: int = 1354 * 2  # roughly the number columns in a 500m dataset

FILTERS = {
    "day_only": {
        "standard_name": [
            "toa_bidirectional_reflectance",
            "corrected_reflectance",
            "true_color",
            "false_color",
            "natural_color",
        ],
    },
    "night_only": {
        "standard_name": ["temperature_difference"],
    },
}

PRODUCT_ALIASES = {}
DEFAULTS = []
VIS_PRODUCTS = []
for chan_num in list(range(1, 8)) + [26]:
    p2g_name = f"vis{chan_num:02d}"
    VIS_PRODUCTS.append(p2g_name)
    PRODUCT_ALIASES[p2g_name] = DataQuery(name=f"{chan_num}", calibration="reflectance")
    DEFAULTS.append(p2g_name)

BT_PRODUCTS = []
RAD_PRODUCTS = []
for chan_num in list(range(20, 26)) + list(range(27, 37)):
    p2g_name = f"bt{chan_num:02d}"
    BT_PRODUCTS.append(p2g_name)
    PRODUCT_ALIASES[p2g_name] = DataQuery(name=f"{chan_num}", calibration="brightness_temperature")
    DEFAULTS.append(p2g_name)

    p2g_name = f"ir{chan_num:02d}"
    RAD_PRODUCTS.append(p2g_name)
    PRODUCT_ALIASES[p2g_name] = DataQuery(name=f"{chan_num}", calibration="radiance")

COMPOSITES = [
    "true_color",
    "false_color",
    "fog",
]
DEFAULTS.extend(COMPOSITES)

_AWIPS_TRUE_COLOR = ["modis_crefl01_250m", "modis_crefl04_250m", "modis_crefl03_250m"]
_AWIPS_FALSE_COLOR = ["modis_crefl07_500m", "modis_crefl02_250m", "modis_crefl01_250m"]


class ReaderProxy(ReaderProxyBase):
    """Provide Polar2Grid-specific information about this reader's products."""

    is_polar2grid_reader = True

    def get_default_products(self) -> list[str]:
        """Get products to load if users hasn't specified any others."""
        return DEFAULTS

    def get_all_products(self) -> list[str]:
        """Get all polar2grid products that could be loaded."""
        return VIS_PRODUCTS + BT_PRODUCTS + RAD_PRODUCTS + COMPOSITES

    @property
    def _aliases(self) -> dict[str, DataQuery]:
        return PRODUCT_ALIASES


def add_reader_argument_groups(
    parser: ArgumentParser, group: Optional[_ArgumentGroup] = None
) -> tuple[Optional[_ArgumentGroup], Optional[_ArgumentGroup]]:
    """Add reader-specific command line arguments to an existing argument parser.

    If ``group`` is provided then arguments are added to this group. If not,
    a new group is added to the parser and arguments added to this new group.

    """
    if group is None:
        group = parser.add_argument_group(title="MODIS L1B Reader")
    group.add_argument(
        "--ir-products",
        dest="products",
        action=ExtendConstAction,
        const=RAD_PRODUCTS,
        help="Add IR products to list of products",
    )
    group.add_argument(
        "--bt-products",
        dest="products",
        action=ExtendConstAction,
        const=BT_PRODUCTS,
        help="Add BT products to list of products",
    )
    group.add_argument(
        "--vis-products",
        dest="products",
        action=ExtendConstAction,
        const=VIS_PRODUCTS,
        help="Add Visible products to list of products",
    )
    group.add_argument(
        "--awips-true-color",
        dest="products",
        action=ExtendConstAction,
        const=_AWIPS_TRUE_COLOR,
        help="Add individual CREFL corrected products to create " "the 'true_color' composite in AWIPS.",
    )
    group.add_argument(
        "--awips-false-color",
        dest="products",
        action=ExtendConstAction,
        const=_AWIPS_FALSE_COLOR,
        help="Add individual CREFL corrected products to create " "the 'false_color' composite in AWIPS.",
    )
    group.add_argument(
        "--mask-saturated",
        default=False,
        action="store_true",
        help=argparse.SUPPRESS,
        # help="Mask saturated band 2 pixels as invalid instead of clipping to max reflectance",
    )
    return group, None



#!/usr/bin/env python
# encoding: utf-8
# Copyright (C) 2021 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Enhancement functions shared between multiple sensors."""

import numpy as np
from satpy.enhancements import colorize as _colorize
from satpy.enhancements import palettize as _palettize

from polar2grid.utils.config import get_polar2grid_home


def temperature_difference(img, min_stretch, max_stretch, **kwargs):
    """Scale data linearly with a buffer on the edges for over limit data.

    Basic behavior is to put -10 to 10 range into 5 to 205 with clipped data
    set to 4 and 206.

    """
    img.crude_stretch(min_stretch, max_stretch)
    # we assume uint8 images for legacy AWIPS comparisons
    offset = 1 / 255.0
    factor = (205.0 - 5.0) / 255.0
    img.data.data = img.data.data * factor
    img.data.data = np.clip(img.data.data, -offset, factor + offset)  # 4 and 206 offset
    img.data.data = img.data.data + 5 * offset  # lower bound of 5
    return img


def _parse_palettes_for_p2g_cmap(palettes: list):
    for palette in palettes:
        filename = palette.get("filename", "")
        if not filename:
            yield palette
            continue
        p2g_home = get_polar2grid_home()
        filename = filename.replace("$POLAR2GRID_HOME", p2g_home)
        filename = filename.replace("$GEO2GRID_HOME", p2g_home)
        # if _is_awips_cmap(filename):
        #     color_arr = _get_awips_colors(filename)
        #     palette["colors"] = color_arr
        #     palette["color_scale"] = 1.0
        #     del palette["filename"]
        # else:
        palette["filename"] = filename
        yield palette


# def _is_awips_cmap(cmap_filename: str) -> bool:
#     if not cmap_filename.endswith(".cmap"):
#         return False
#     with open(cmap_filename, "r") as cmap_file:
#         return "xml" in cmap_file.read()
#
#
# def _get_awips_colors(cmap_file: str) -> list[tuple[float, float, float]]:
#     # TODO: If this is used, add it to the dependencies
#     from defusedxml import ElementTree
#
#     tree = ElementTree.parse(cmap_file)
#     colors = [(float(c.get("r")), float(c.get("g")), float(c.get("b"))) for c in tree.findall("color")]
#     return colors


def colorize(img, **kwargs):
    kwargs["palettes"] = list(_parse_palettes_for_p2g_cmap(kwargs["palettes"]))
    return _colorize(img, **kwargs)


def palettize(img, **kwargs):
    kwargs["palettes"] = list(_parse_palettes_for_p2g_cmap(kwargs["palettes"]))
    return _palettize(img, **kwargs)



#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2022 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""The AGRI FY-4A L1 Reader operates on Level 1 (L1) HDF5 files for the
Advanced Geostationary Radiation Imager (AGRI) instrument on board the
Feng-Yun - 4 (FY-4A) satellite. Geo2Grid determines if it supports files
based on the input filename. Files for AGRI supported by Geo2Grid usually
have the following naming scheme::

    FY4A-_AGRI--_N_DISK_1047E_L1-_FDI-_MULT_NOM_20220117000000_20220117001459_0500M_V0001.HDF

The AGRI L1 reader supports all instrument spectral bands, identified in
Geo2Grid as the products shown in the table below. The
AGRI L1 reader for FY-4A can be provided to the main geo2grid.sh script
using the ``-r`` option and the reader name ``agri_fy4a_l1``.

The list of supported products includes true and natural color imagery.
These are created by means of a python based atmospheric Rayleigh
scattering correction algorithm that is executed as part of the |project| AGRI
L1 reader, along with sharpening to the highest spatial resolution. For
more information on the creation of RGBs, please see the
:ref:`RGB section <getting_started_rgb>`.

+---------------------------+-----------------------------------------------------+
| **Product Name**          | **Description**                                     |
+===========================+=====================================================+
| C01                       | Channel 1 (0.47um) Reflectance Band                 |
+---------------------------+-----------------------------------------------------+
| C02                       | Channel 2 (0.65um) Reflectance Band                 |
+---------------------------+-----------------------------------------------------+
| C03                       | Channel 3 (0.83um) Reflectance Band                 |
+---------------------------+-----------------------------------------------------+
| C04                       | Channel 4 (1.37um) Reflectance Band                 |
+---------------------------+-----------------------------------------------------+
| C05                       | Channel 5 (1.61um) Reflectance Band                 |
+---------------------------+-----------------------------------------------------+
| C06                       | Channel 6 (2.25um) Reflectance Band                 |
+---------------------------+-----------------------------------------------------+
| C07                       | Channel 7 (3.75um) Brightness Temperature Band      |
+---------------------------+-----------------------------------------------------+
| C08                       | Channel 8 (3.75um) Brightness Temperature Band      |
+---------------------------+-----------------------------------------------------+
| C09                       | Channel 9 (6.25um) Brightness Temperature Band      |
+---------------------------+-----------------------------------------------------+
| C10                       | Channel 10 (7.10um) Brightness Temperature Band     |
+---------------------------+-----------------------------------------------------+
| C11                       | Channel 11 (8.5um) Brightness Temperature Band      |
+---------------------------+-----------------------------------------------------+
| C12                       | Channel 12 (10.7um) Brightness Temperature Band     |
+---------------------------+-----------------------------------------------------+
| C13                       | Channel 13 (12.0um) Brightness Temperature Band     |
+---------------------------+-----------------------------------------------------+
| C14                       | Channel 14 (13.5um) Brightness Temperature Band     |
+---------------------------+-----------------------------------------------------+
| true_color                | Ratio sharpened rayleigh corrected true color       |
+---------------------------+-----------------------------------------------------+
| natural_color             | Ratio sharpened corrected natural color             |
+---------------------------+-----------------------------------------------------+
| ash                       | Ash RGB                                             |
+---------------------------+-----------------------------------------------------+
| dust                      | Dust RGB                                            |
+---------------------------+-----------------------------------------------------+
| fog                       | Fog RGB                                             |
+---------------------------+-----------------------------------------------------+


"""

from __future__ import annotations

from argparse import ArgumentParser, _ArgumentGroup
from typing import Optional

from ._base import ReaderProxyBase

PREFERRED_CHUNK_SIZE: int = 4096

READER_PRODUCTS = ["C{:02d}".format(x) for x in range(1, 15)]
COMPOSITE_PRODUCTS = [
    "true_color",
    "natural_color",
    "ash",
    "dust",
    "fog",
]


class ReaderProxy(ReaderProxyBase):
    """Provide Polar2Grid-specific information about this reader's products."""

    is_geo2grid_reader: bool = True

    def get_default_products(self) -> list[str]:
        """Get products to load if users hasn't specified any others."""
        return READER_PRODUCTS + COMPOSITE_PRODUCTS[:2]

    def get_all_products(self) -> list[str]:
        """Get all polar2grid products that could be loaded."""
        return READER_PRODUCTS + COMPOSITE_PRODUCTS


def add_reader_argument_groups(
    parser: ArgumentParser, group: Optional[_ArgumentGroup] = None
) -> tuple[Optional[_ArgumentGroup], Optional[_ArgumentGroup]]:
    """Add reader-specific command line arguments to an existing argument parser.

    If ``group`` is provided then arguments are added to this group. If not,
    a new group is added to the parser and arguments added to this new group.

    """
    if group is None:
        group = parser.add_argument_group(title="AGRI FY-4B Level 1 Reader")
    return group, None



# Copyright (C) 2023 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""The VIIRS EDR Reader operates on Environmental Data Record (SDR) HDF5 files
from the Suomi National Polar-orbiting Partnership's (NPP), the NOAA20, or the
NOAA-21 Visible/Infrared Imager Radiometer Suite (VIIRS) instrument. The VIIRS
EDR reader requires filenames to match one of a couple different standard
filename schemes used for official products. EDR files are typically named
as below::

    JRR-AOD_v3r2_npp_s202307230941369_e202307230943011_c202307231038217.nc

The VIIRS EDR reader supports most JPSS Risk Reduction (JRR) enterprise algorithm
output. The VIIRS EDR reader can be specified to the ``polar2grid.sh`` script
with the reader name ``viirs_edr``.

This reader's default remapping algorithm is ``ewa`` for Elliptical Weighted
Averaging resampling. The ``--weight-delta-max`` parameter set to 40 and the
``--weight-distance-max`` parameter set to 2.

+---------------------------+-----------------------------------------------------+
| Product Name              | Description                                         |
+===========================+=====================================================+
| surf_refl_i01             | I01 Surface Reflectance Band                        |
+---------------------------+-----------------------------------------------------+
| surf_refl_i02             | I02 Surface Reflectance Band                        |
+---------------------------+-----------------------------------------------------+
| surf_refl_i03             | I03 Surface Reflectance Band                        |
+---------------------------+-----------------------------------------------------+
| surf_refl_m01             | M01 Surface Reflectance Band                        |
+---------------------------+-----------------------------------------------------+
| surf_refl_m02             | M02 Surface Reflectance Band                        |
+---------------------------+-----------------------------------------------------+
| surf_refl_m03             | M03 Surface Reflectance Band                        |
+---------------------------+-----------------------------------------------------+
| surf_refl_m04             | M04 Surface Reflectance Band                        |
+---------------------------+-----------------------------------------------------+
| surf_refl_m05             | M05 Surface Reflectance Band                        |
+---------------------------+-----------------------------------------------------+
| surf_refl_m07             | M07 Surface Reflectance Band                        |
+---------------------------+-----------------------------------------------------+
| surf_refl_m08             | M08 Surface Reflectance Band                        |
+---------------------------+-----------------------------------------------------+
| surf_refl_m10             | M10 Surface Reflectance Band                        |
+---------------------------+-----------------------------------------------------+
| surf_refl_m11             | M11 Surface Reflectance Band                        |
+---------------------------+-----------------------------------------------------+
| true_color_surf           | Ratio sharpened surface true color                  |
+---------------------------+-----------------------------------------------------+
| false_color_surf          | Ratio sharpened surface false color                 |
+---------------------------+-----------------------------------------------------+
| NDVI                      | Normalized Difference Vegetation Index              |
+---------------------------+-----------------------------------------------------+
| EVI                       | Enhanced Vegetation Index                           |
+---------------------------+-----------------------------------------------------+
| CldTopTemp                | Cloud Top Temperature                               |
+---------------------------+-----------------------------------------------------+
| CldTopHght                | Cloud Top Height                                    |
+---------------------------+-----------------------------------------------------+
| AOD550                    | Aerosol Optical Depth                               |
+---------------------------+-----------------------------------------------------+
| VLST                      | Land Surface Temperature                            |
+---------------------------+-----------------------------------------------------+

"""

from __future__ import annotations

from argparse import ArgumentParser, _ArgumentGroup, BooleanOptionalAction
from typing import Optional

from satpy import DataQuery

from ._base import ReaderProxyBase

PREFERRED_CHUNK_SIZE: int = 6400

I_PRODUCTS = ["surf_refl_I{:02d}".format(chan_num) for chan_num in range(1, 4)]
M_PRODUCTS = ["surf_refl_M{:02d}".format(chan_num) for chan_num in range(1, 12) if chan_num not in (6, 9)]
SURF_COMPS = ["true_color_surf", "false_color_surf"]
OTHER_PRODS = ["NDVI", "EVI", "CldTopTemp", "CldTopHght", "AOD550", "VLST"]

PRODUCT_ALIASES = {}
SURF_ALIASES = []

for surf_band in I_PRODUCTS + M_PRODUCTS:
    PRODUCT_ALIASES[surf_band.lower()] = surf_band
    SURF_ALIASES.append(surf_band.lower())

DEFAULT_PRODUCTS = SURF_ALIASES + SURF_COMPS + OTHER_PRODS
P2G_PRODUCTS = SURF_ALIASES + SURF_COMPS + OTHER_PRODS

FILTERS = {
    "day_only": {
        "standard_name": [
            "surface_bidirectional_reflectance",
            "true_color",
            "false_color",
            "natural_color",
        ],
    },
    "night_only": {},
}


class ReaderProxy(ReaderProxyBase):
    """Provide Polar2Grid-specific information about this reader's products."""

    is_polar2grid_reader = True

    def get_default_products(self) -> list[str]:
        """Get products to load if users hasn't specified any others."""
        return DEFAULT_PRODUCTS

    def get_all_products(self):
        """Get all polar2grid products that could be loaded."""
        return P2G_PRODUCTS

    @property
    def _aliases(self) -> dict[str, DataQuery | str]:
        return PRODUCT_ALIASES


def add_reader_argument_groups(
    parser: ArgumentParser, group: Optional[_ArgumentGroup] = None
) -> tuple[Optional[_ArgumentGroup], Optional[_ArgumentGroup]]:
    """Add reader-specific command line arguments to an existing argument parser.

    If ``group`` is provided then arguments are added to this group. If not,
    a new group is added to the parser and arguments added to this new group.

    """
    if group is None:
        group = parser.add_argument_group(title="VIIRS EDR Reader")

    group.add_argument(
        "--filter-veg",
        action=BooleanOptionalAction,
        default=True,
        help="Filter vegetation index variables by various quality flags. Default is enabled.",
    )
    group.add_argument(
        "--aod-qc-filter",
        default=1,
        type=_int_or_none,
        choices=[None, 0, 1, 2, 3],
        help="Filter AOD550 variable by QCAll variable. Value specifies the maximum "
        "quality to include (0-high, 1-medium, 2-low, 3-no retrieval). Defaults to "
        "1 which means medium and high quality data are included.",
    )

    return group, None


def _int_or_none(value: str) -> None | int:
    if value.lower() == "none":
        return None
    return int(value)



#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2021 Space Science and Engineering Center (SSEC),
# University of Wisconsin-Madison.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Configure behave tests."""

import os
import shutil
import sys
import tempfile


def before_all(context):
    context.html_dst = context.config.userdata.get("html_dst", None)
    if context.html_dst is not None:
        context.html_dst = os.path.join(context.html_dst, "test_status")
    context.datapath = context.config.userdata["datapath"]
    if not context.datapath.startswith(os.sep):
        context.datapath = os.path.join(os.getcwd(), context.datapath)
    os.environ["DATA_PATH"] = context.datapath  # for test command replacement
    p2g_home = os.environ.get("POLAR2GRID_HOME")
    # if POLAR2GRID_HOME isn't defined, assume things are installed in the python prefix
    context.p2g_path = os.path.join(p2g_home, "bin") if p2g_home is not None else os.path.join(sys.prefix, "bin")
    if p2g_home is None:
        # assume development/editable install
        import polar2grid

        os.environ["POLAR2GRID_HOME"] = os.path.join(os.path.dirname(polar2grid.__file__), "..", "swbundle")


def before_feature(context, feature):
    tmpdir = tempfile.gettempdir()
    feature_name = os.path.basename(feature.filename).replace("<", "").replace(">", "").replace(".feature", "")
    context.base_temp_dir = os.path.join(tmpdir, "p2g_integration_tests", feature_name)

    # remove any previous test results
    if os.path.isdir(context.base_temp_dir):
        shutil.rmtree(context.base_temp_dir, ignore_errors=True)
    os.makedirs(context.base_temp_dir, exist_ok=True)


def after_all(context):
    pass



#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2022 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Tests for the __main__.py script."""

import pytest

from polar2grid.__main__ import g2g_main, p2g_main


@pytest.mark.parametrize(
    ("main_func", "exp_reader"),
    [
        (p2g_main, "viirs_sdr"),
        (g2g_main, "abi_l1b"),
    ],
)
def test_main_call(main_func, exp_reader, capsys):
    with pytest.raises(SystemExit) as e:
        main_func(["-h"])
    assert e.value.code == 0
    sout = capsys.readouterr()
    assert exp_reader in sout.out


def test_call_main_script():
    import subprocess

    subprocess.check_call(["python3", "-m", "polar2grid", "-h"])



#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2022 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Tests for the compare.py script."""

import os
from glob import glob

import numpy as np
import pytest

from polar2grid.utils.warnings import ignore_no_georef

SHAPE1 = (200, 100)
SHAPE2 = (200, 101)
IMAGE1_L_UINT8_ZEROS = np.zeros(SHAPE1, dtype=np.uint8)
IMAGE2_L_UINT8_ZEROS = np.zeros(SHAPE2, dtype=np.uint8)
IMAGE3_L_UINT8_ONES = np.ones(SHAPE1, dtype=np.uint8)
# don't use a shape divisible by 64-bit float
IMAGE4_L_FLOAT32_ZEROS = np.zeros((SHAPE1[0] + 1, SHAPE1[1] + 1), dtype=np.float32)
IMAGE4_RGB_UINT8_ZEROS = np.zeros((3,) + SHAPE1, dtype=np.uint8)
IMAGE5_RGB_UINT8_ZEROS = np.zeros((3,) + SHAPE2, dtype=np.uint8)
IMAGE6_RGBA_UINT8_ZEROS = np.zeros((4,) + SHAPE1, dtype=np.uint8)
IMAGE7_RGB_UINT8_ONES = np.ones((3,) + SHAPE1, dtype=np.uint8)
IMAGE_LIST1 = [IMAGE1_L_UINT8_ZEROS, IMAGE1_L_UINT8_ZEROS]
IMAGE_LIST2 = [IMAGE1_L_UINT8_ZEROS, IMAGE2_L_UINT8_ZEROS, IMAGE4_RGB_UINT8_ZEROS]


def _create_geotiffs(base_dir, img_data):
    import rasterio

    if not isinstance(img_data, (list, tuple)):
        img_data = [img_data]

    for idx, img_arr in enumerate(img_data):
        band_count = 1 if img_arr.ndim == 2 else img_arr.shape[0]
        gtiff_fn = os.path.join(base_dir, f"test{idx}.tif")
        with ignore_no_georef():
            with rasterio.open(
                gtiff_fn,
                "w",
                driver="GTiff",
                count=band_count,
                height=img_arr.shape[-2],
                width=img_arr.shape[-1],
                dtype=img_arr.dtype,
            ) as gtiff_file:
                if img_arr.ndim == 2:
                    img_arr = img_arr[None, :, :]
                for band_idx, band_arr in enumerate(img_arr):
                    gtiff_file.write(band_arr, band_idx + 1)


def _create_hdf5(base_dir, img_data):
    import h5py

    if not isinstance(img_data, (list, tuple)):
        img_data = [img_data]

    h5_fn = os.path.join(base_dir, "test.h5")
    with h5py.File(h5_fn, "w") as h:
        for idx, img_arr in enumerate(img_data):
            ds = h.create_dataset(f"image{idx}", data=img_arr)
            ds.attrs["some_attr"] = f"some_value{idx}"


def _create_hdf5_with_groups(base_dir, img_data):
    import h5py

    if not isinstance(img_data, (list, tuple)):
        img_data = [img_data]

    h5_fn = os.path.join(base_dir, "test.h5")
    with h5py.File(h5_fn, "w") as h:
        group = h.create_group("wgs84_fit")
        for idx, img_arr in enumerate(img_data):
            ds = group.create_dataset(f"image{idx}", data=img_arr)
            ds.attrs["some_attr"] = f"some_value{idx}"


def _create_binaries(base_dir, img_data):
    if not isinstance(img_data, (list, tuple)):
        img_data = [img_data]
    for idx, img_arr in enumerate(img_data):
        bin_fn = os.path.join(base_dir, f"test{idx}.dat")
        img_arr.tofile(bin_fn)


def _create_awips_tiled(base_dir, img_data):
    from netCDF4 import Dataset

    if not isinstance(img_data, (list, tuple)):
        img_data = [img_data]
    # 2 tiles per dataset
    for idx, img_arr in enumerate(img_data):
        for tile_id in ("T01", "T02"):
            fn = f"SSEC_AII_gcom-w1_amsr2_image{idx}_LCC_T{tile_id}_20160719_1903.nc"
            fp = os.path.join(base_dir, fn)
            nc = Dataset(fp, "w")
            nc.createDimension("y", img_arr.shape[-2])
            nc.createDimension("x", img_arr.shape[-1])
            dims = ("y", "x")
            if img_arr.ndim == 3:
                dims = ("bands",) + dims
                nc.createDimension("bands", img_arr.shape[0])
            nc_var = nc.createVariable("data", img_arr.dtype, dimensions=dims)
            nc_var[:] = img_arr
            nc_var.grid_mapping = "lcc_grid_mapping"

            lcc_grid_mapping = nc.createVariable("lcc_grid_mapping", np.int32)
            lcc_grid_mapping.grid_mapping_name = "lambert_conformal_conic"
            lcc_grid_mapping.standard_parallel = 25.0
            lcc_grid_mapping.longitude_of_central_meridian = 0.0
            lcc_grid_mapping.latitude_of_projection_origin = 35.0

            y_var = nc.createVariable("y", np.float32, dimensions=("y",))
            y_var[:] = np.arange(img_arr.shape[-2], dtype=np.float32)
            x_var = nc.createVariable("x", np.float32, dimensions=("x",))
            x_var[:] = np.arange(img_arr.shape[-1], dtype=np.float32)


@pytest.mark.parametrize(
    ("expected_file_func", "actual_file_func"),
    [
        (None, None),  # no files
        (_create_geotiffs, _create_hdf5),
        (_create_geotiffs, _create_geotiffs),
        (_create_hdf5, _create_hdf5),
        (_create_hdf5_with_groups, _create_hdf5_with_groups),
        (_create_binaries, _create_binaries),
        (_create_awips_tiled, _create_awips_tiled),
    ],
)
@pytest.mark.parametrize(
    ("expected_data", "actual_data", "exp_num_diff"),
    [
        (IMAGE1_L_UINT8_ZEROS, IMAGE1_L_UINT8_ZEROS, 0),
        (IMAGE1_L_UINT8_ZEROS, IMAGE2_L_UINT8_ZEROS, 1),
        (IMAGE1_L_UINT8_ZEROS, IMAGE3_L_UINT8_ONES, 1),
        (IMAGE3_L_UINT8_ONES, IMAGE3_L_UINT8_ONES, 0),
        (IMAGE4_RGB_UINT8_ZEROS, IMAGE4_RGB_UINT8_ZEROS, 0),
        (IMAGE6_RGBA_UINT8_ZEROS, IMAGE6_RGBA_UINT8_ZEROS, 0),
        (IMAGE4_RGB_UINT8_ZEROS, IMAGE5_RGB_UINT8_ZEROS, 1),
        (IMAGE4_RGB_UINT8_ZEROS, IMAGE6_RGBA_UINT8_ZEROS, 1),
        (IMAGE4_RGB_UINT8_ZEROS, IMAGE7_RGB_UINT8_ONES, 1),
        (IMAGE_LIST1, IMAGE_LIST1, 0),
        (IMAGE_LIST2, IMAGE_LIST2, 0),
        (IMAGE_LIST2, IMAGE_LIST1, 2),
        (IMAGE4_L_FLOAT32_ZEROS, IMAGE4_L_FLOAT32_ZEROS, 0),
    ],
)
@pytest.mark.parametrize("include_html", [False, True])
def test_basic_compare(
    tmp_path, expected_file_func, actual_file_func, expected_data, actual_data, exp_num_diff, include_html
):
    from polar2grid.compare import main

    expected_dir = tmp_path / "expected"
    expected_dir.mkdir()
    actual_dir = tmp_path / "actual"
    actual_dir.mkdir()

    if expected_file_func:
        expected_file_func(expected_dir, expected_data)
    if actual_file_func:
        actual_file_func(actual_dir, actual_data)

    margin_of_error = 81231 / 1514041.44

    args = [
        str(expected_dir),
        str(actual_dir),
        "--margin-of-error",
        str(margin_of_error),
    ]
    dtype = None
    if dtype is not None:
        args.extend(["--dtype", dtype])
    html_file = tmp_path / "test_output.html"
    if include_html:
        args.extend(["--html", str(html_file)])

    with ignore_no_georef():
        num_diff_files = main(args)
    exp_num_png_files = _get_exp_num_png_files(actual_data, expected_data, expected_file_func)
    _check_num_diff_files(num_diff_files, exp_num_diff, expected_file_func, actual_file_func)
    _check_html_output(include_html, html_file, exp_num_png_files, expected_file_func, actual_file_func)


def _get_exp_num_png_files(actual_data, expected_data, expected_file_func):
    exp_num_expected_png_files = len(expected_data) if isinstance(expected_data, list) else 1
    exp_num_actual_png_files = len(actual_data) if isinstance(actual_data, list) else 1
    if _is_multivar_format(expected_file_func):
        # if the file exists then we will be creating a thumbnail for every possible variable
        exp_num_png_files = exp_num_actual_png_files + exp_num_expected_png_files
    else:
        # images are only generated when both files exist
        exp_num_png_files = min(exp_num_expected_png_files, exp_num_actual_png_files) * 2
    exp_num_png_files *= _files_per_variable(expected_file_func) * (1 + _nonvariable_png_files(expected_file_func))
    return exp_num_png_files


def _check_num_diff_files(num_diff_files, exp_num_diff, expected_file_func, actual_file_func):
    have_files = actual_file_func is not None and expected_file_func is not None
    if not have_files:
        assert num_diff_files == 0
    elif actual_file_func is not expected_file_func:
        assert num_diff_files != 0
    elif _is_multivar_format(expected_file_func):
        # multiple variables in one file
        assert num_diff_files == (1 if exp_num_diff > 0 else 0)
    else:
        assert num_diff_files == exp_num_diff * _files_per_variable(expected_file_func)


def _nonvariable_png_files(expected_file_func):
    """Get number of PNG files that are produced from ancillary data in the files."""
    return {
        _create_awips_tiled: 2,
    }.get(expected_file_func, 0)


def _is_multivar_format(expected_file_func):
    return {
        _create_hdf5: True,
        _create_hdf5_with_groups: True,
    }.get(expected_file_func, False)


def _files_per_variable(expected_file_func):
    return {_create_awips_tiled: 2}.get(expected_file_func, 1)


def _check_html_output(include_html, html_file, exp_total_files, expected_file_func, actual_file_func):
    base_dir = os.path.dirname(html_file)
    if include_html:
        assert os.path.isfile(html_file)
        img_glob = os.path.join(base_dir, "_images", "*.png")
        formats_are_different = actual_file_func is not expected_file_func
        files_were_generated = actual_file_func is not None and expected_file_func is not None
        can_gen_tn = _can_generate_thumbnails(actual_file_func)
        should_have_thumbnails = not formats_are_different and files_were_generated and can_gen_tn

        if should_have_thumbnails:
            assert len(glob(img_glob)) == exp_total_files
        else:
            assert len(glob(img_glob)) == 0
    else:
        assert len(glob(os.path.join(base_dir, "*.html"))) == 0


def _can_generate_thumbnails(creation_func) -> bool:
    return creation_func in (
        _create_geotiffs,
        _create_awips_tiled,
        _create_hdf5,
        _create_hdf5_with_groups,
        _create_binaries,
    )



#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2019 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""The VIIRS EDR FLOOD reader operates on HDF4 files.

Files usually have the following naming scheme:

    WATER_VIIRS_Prj_SVI_{platform_shortname}_d{start_time:%Y%m%d_t%H%M%S%f}_e{end_time:%H%M%S%f}_b{orbit:5d}_{source:8s}_{dim0}_{dim1}_01.hdf
    or
    WATER_VIIRS_Prj_SVI_{platform_shortname}_d{start_time:%Y%m%d_t%H%M%S%f}_e{end_time:%H%M%S%f}_b{orbit:5d}_{source:8s}_{aoi:3s}_{dim0}_{dim1}_01.hdf

This reader's default resampling algorithm is ``nearest`` for Nearest Neighbor resampling.
The ``--remap_method`` parameter is set to ``nearest``.

+---------------------------+-----------------------------------------------------+
| **Product Name**          | **Description**                                     |
+===========================+=====================================================+
| WaterDetection            | Channel 1 Reflectance Band                          |
+---------------------------+-----------------------------------------------------+
"""

from __future__ import annotations

from argparse import ArgumentParser, _ArgumentGroup
from typing import Optional

from satpy import DataQuery

from ._base import ReaderProxyBase

DEFAULT_DATASETS = ["WaterDetection"]


class ReaderProxy(ReaderProxyBase):
    """Provide Polar2Grid-specific information about this reader's products."""

    is_polar2grid_reader = True

    def get_default_products(self) -> list[str]:
        """Get products to load if users hasn't specified any others."""
        return DEFAULT_DATASETS

    def get_all_products(self) -> list[str]:
        """Get all polar2grid products that could be loaded."""
        return DEFAULT_DATASETS

    @property
    def _aliases(self) -> dict[str, DataQuery]:
        return {}


def add_reader_argument_groups(
    parser: ArgumentParser, group: Optional[_ArgumentGroup] = None
) -> tuple[Optional[_ArgumentGroup], Optional[_ArgumentGroup]]:
    """Add reader-specific command line arguments to an existing argument parser.

    If ``group`` is provided then arguments are added to this group. If not,
    a new group is added to the parser and arguments added to this new group.

    """
    if group is None:
        group = parser.add_argument_group(title="VIIRS EDR Active Fires Reader")
    return group, None



# Copyright (C) 2023 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""The FCI Level 1c Reader operates on EUMETSAT Level 1c NetCDF files
from the Meteosat Third Generation (MTG) Flexible Combined Imager (FCI)
instrument. The FCI L1c reader works off of the input filenames
to determine if a file is supported by Geo2Grid. Files usually have the
following naming scheme::

    W_XX-EUMETSAT-Darmstadt,IMG+SAT,MTI1+FCI-1C-RRAD-FDHSI-FD--CHK-BODY---NC4E_C_EUMT_20170920115515_GTT_DEV_20170920115008_20170920115015_N__T_0072_0001.nc

The FCI L1c reader supports all instrument spectral bands, identified in
Geo2Grid as the products shown in the table below. The
reader can be provided to the main geo2grid.sh script
using the ``-r`` option and the reader name ``fci_l1c_nc``.

+---------------------------+-----------------------------------------------------+
| **Product Name**          | **Description**                                     |
+===========================+=====================================================+
| vis_04                    | 0.44um Reflectance Band                             |
+---------------------------+-----------------------------------------------------+
| vis_05                    | 0.51um Reflectance Band                             |
+---------------------------+-----------------------------------------------------+
| vis_06                    | 0.64um Reflectance Band                             |
+---------------------------+-----------------------------------------------------+
| vis_08                    | 0.86um Reflectance Band                             |
+---------------------------+-----------------------------------------------------+
| vis_09                    | 0.91um Reflectance Band                             |
+---------------------------+-----------------------------------------------------+
| nir_13                    | 1.3um Near-IR Reflectance Band                      |
+---------------------------+-----------------------------------------------------+
| nir_16                    | 1.6um Near-IR Reflectance Band                      |
+---------------------------+-----------------------------------------------------+
| nir_22                    | 2.2um Near-IR Reflectance Band                      |
+---------------------------+-----------------------------------------------------+
| ir_38                     | 3.8um Brightness Temperature Band                   |
+---------------------------+-----------------------------------------------------+
| wv_63                     | 6.3um Water Vapor Brightness Temperature Band       |
+---------------------------+-----------------------------------------------------+
| wv_73                     | 7.3um Water Vapor Brightness Temperature Band       |
+---------------------------+-----------------------------------------------------+
| ir_87                     | 8.7um Brightness Temperature Band                   |
+---------------------------+-----------------------------------------------------+
| ir_97                     | 9.7um Brightness Temperature Band                   |
+---------------------------+-----------------------------------------------------+
| ir_105                    | 10.5um Brightness Temperature Band                  |
+---------------------------+-----------------------------------------------------+
| ir_123                    | 12.3um Brightness Temperature Band                  |
+---------------------------+-----------------------------------------------------+
| ir_133                    | 13.3um Brightness Temperature Band                  |
+---------------------------+-----------------------------------------------------+
| true_color                | Ratio sharpened rayleigh corrected true color       |
+---------------------------+-----------------------------------------------------+
| natural_color             | Ratio sharpened corrected natural color             |
+---------------------------+-----------------------------------------------------+

"""

from __future__ import annotations

from argparse import ArgumentParser, _ArgumentGroup
from typing import Optional

from ._base import ReaderProxyBase

PREFERRED_CHUNK_SIZE: int = 1024

READER_PRODUCTS = [
    "vis_04",
    "vis_05",
    "vis_06",
    "vis_08",
    "vis_09",
    "nir_13",
    "nir_16",
    "nir_22",
    "ir_38",
    "wv_63",
    "wv_73",
    "ir_87",
    "ir_97",
    "ir_105",
    "ir_123",
    "ir_133",
]
COMPOSITE_PRODUCTS = [
    "true_color",
    "natural_color",
]


class ReaderProxy(ReaderProxyBase):
    """Provide Polar2Grid-specific information about this reader's products."""

    is_geo2grid_reader = True

    def get_default_products(self) -> list[str]:
        """Get products to load if users hasn't specified any others."""
        return READER_PRODUCTS + COMPOSITE_PRODUCTS[:2]

    def get_all_products(self) -> list[str]:
        """Get all polar2grid products that could be loaded."""
        return READER_PRODUCTS + COMPOSITE_PRODUCTS


def add_reader_argument_groups(
    parser: ArgumentParser, group: Optional[_ArgumentGroup] = None
) -> tuple[Optional[_ArgumentGroup], Optional[_ArgumentGroup]]:
    """Add reader-specific command line arguments to an existing argument parser.

    If ``group`` is provided then arguments are added to this group. If not,
    a new group is added to the parser and arguments added to this new group.

    """
    if group is None:
        group = parser.add_argument_group(title="FCI L1c NetCDF Reader")
    load_group = parser.add_argument_group(title="FCI L1c NetCDF Reader Loading")
    load_group.add_argument(
        "--upper-right-corner",
        default="NE",
        help="Orientation of the data. Data in files is 'SE' for South-East "
        "which is upside down from the traditional 'north is up'. This "
        "argument defaults to 'NE' for North-East and produces the "
        "traditional north-up image orientation.",
    )
    return group, load_group



#!/usr/bin/env python
# encoding: utf-8
# Copyright (C) 2021 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Tests for resampling a Scene."""

from unittest import mock

import dask
import pytest
from pyresample.geometry import SwathDefinition
from pytest_lazy_fixtures import lf as lazy_fixture
from satpy import Scene
from satpy.resample import DaskEWAResampler, KDTreeResampler, NativeResampler
from satpy.tests.utils import CustomScheduler

from polar2grid.resample._resample_scene import resample_scene


@pytest.mark.parametrize(
    (
        "input_scene",
        "grids",
        "grid_configs",
        "resampler",
        "exp_names",
        "max_computes",
        "is_polar2grid",
        "exp_resampler_cls",
        "exp_cov_filter_count",
        "extra_kwargs",
        "exp_kwargs",
    ),
    [
        (
            lazy_fixture("viirs_sdr_i01_scene"),
            ["wgs84_fit"],
            lazy_fixture("builtin_grids_yaml"),
            None,
            ["I01"],
            1,
            True,
            DaskEWAResampler,
            0,
            {},
            {"weight_delta_max": 40.0, "weight_distance_max": 2.0},
        ),
        (
            lazy_fixture("viirs_sdr_i01_scene"),
            ["wgs84_fit"],
            [],
            None,
            ["I01"],
            1,
            True,
            DaskEWAResampler,
            0,
            {},
            {"weight_delta_max": 40.0, "weight_distance_max": 2.0},
        ),
        (
            lazy_fixture("viirs_sdr_i01_scene"),
            ["211e"],
            [],
            None,
            ["I01"],
            1,
            True,
            DaskEWAResampler,
            1,
            {"grid_coverage": 0.05},
            {"weight_delta_max": 40.0, "weight_distance_max": 2.0},
        ),
        (
            lazy_fixture("viirs_sdr_i01_scene"),
            ["wgs84_fit"],
            ["grids.conf"],
            None,
            ["I01"],
            1,
            True,
            DaskEWAResampler,
            0,
            {},
            {"weight_delta_max": 40.0, "weight_distance_max": 2.0},
        ),
        (
            lazy_fixture("viirs_sdr_i01_scene"),
            ["wgs84_fit"],
            ["grids.conf"],
            "ewa",
            ["I01"],
            1,
            True,
            DaskEWAResampler,
            0,
            {"weight_distance_max": 3.0, "maximum_weight_mode": True},
            {"weight_delta_max": 40.0, "weight_distance_max": 3.0, "maximum_weight_mode": True},
        ),
        (
            lazy_fixture("viirs_sdr_i01_scene"),
            ["wgs84_fit"],
            ["grids.conf"],
            None,
            ["I01"],
            1,
            True,
            DaskEWAResampler,
            0,
            {"weight_distance_max": 3.0, "maximum_weight_mode": True},
            {"weight_delta_max": 40.0, "weight_distance_max": 3.0, "maximum_weight_mode": True},
        ),
        (
            lazy_fixture("abi_l1b_c01_scene"),
            ["wgs84_fit"],
            lazy_fixture("builtin_grids_yaml"),
            "nearest",
            ["C01"],
            0,
            False,
            KDTreeResampler,
            0,
            {},
            {},
        ),
        (
            lazy_fixture("abi_l1b_c01_scene"),
            ["wgs84_fit"],
            lazy_fixture("builtin_grids_yaml"),
            None,
            ["C01"],
            0,
            False,
            KDTreeResampler,
            0,
            {},
            {},
        ),
        (
            lazy_fixture("abi_l1b_c01_scene"),
            ["MAX"],
            [],
            None,
            ["C01"],
            0,
            False,
            NativeResampler,
            0,
            {},
            {},
        ),
        (
            lazy_fixture("avhrr_l1b_1_scene"),
            ["wgs84_fit"],
            lazy_fixture("builtin_grids_yaml"),
            None,
            ["1"],
            1,
            True,
            DaskEWAResampler,
            0,
            {},
            {"weight_delta_max": 10.0, "weight_distance_max": 1.0, "rows_per_scan": 0},
        ),
    ],
)
def test_resample_single_result_per_grid(
    input_scene,
    grids,
    grid_configs,
    resampler,
    exp_cov_filter_count,
    exp_names,
    max_computes,
    is_polar2grid,
    exp_resampler_cls,
    extra_kwargs,
    exp_kwargs,
):
    from satpy.resample import resample

    from polar2grid.filters.resample_coverage import ResampleCoverageFilter

    with (
        dask.config.set(scheduler=CustomScheduler(max_computes)),
        mock.patch("satpy.resample.resample", wraps=resample) as satpy_resample,
        mock.patch(
            "polar2grid.resample._resample_scene.ResampleCoverageFilter", wraps=ResampleCoverageFilter
        ) as resamp_cov,
    ):
        input_scene.load(exp_names)
        scenes_to_save = resample_scene(
            input_scene,
            grids,
            grid_configs,
            resampler,
            is_polar2grid=is_polar2grid,
            **extra_kwargs,
        )
    assert resamp_cov.call_count == exp_cov_filter_count
    satpy_resample.assert_called_once()
    satpy_resample.assert_called_once_with(
        mock.ANY, mock.ANY, mock.ANY, fill_value=mock.ANY, resampler=mock.ANY, **exp_kwargs
    )
    resample_kwargs = satpy_resample.call_args.kwargs
    assert isinstance(resample_kwargs["resampler"], exp_resampler_cls)
    assert len(scenes_to_save) == len(grids)
    for scene, data_ids_set in scenes_to_save:
        assert isinstance(scene, Scene)
        assert len(data_ids_set) == len(exp_names)
        id_names = [x["name"] for x in data_ids_set]
        for exp_name in exp_names:
            assert exp_name in id_names


def test_partial_filter(viirs_sdr_i01_data_array):
    """Test that resampling completes even when coverage filters some datasets."""
    # make another DataArray that is shifted away from the target area
    # orig_lons, orig_lats = viirs_sdr_i01_scene['I01'].attrs['area'].get_lonlats()
    new_i01 = viirs_sdr_i01_data_array.copy()
    orig_lons = new_i01.attrs["area"].lons
    orig_lats = new_i01.attrs["area"].lats
    new_lons = orig_lons + 180.0
    new_lons.attrs = orig_lons.attrs.copy()
    new_swath_def = SwathDefinition(new_lons, orig_lats)
    new_i01.attrs["name"] = "I01_2"
    new_i01.attrs["area"] = new_swath_def
    new_i01.attrs["sensor"] = {"viirs", "modis"}
    new_scn = Scene()
    new_scn["I01"] = viirs_sdr_i01_data_array
    new_scn["I01_2"] = new_i01

    # computation 1: input swath 1 polygon
    # computation 2: input swath 2 polygon (filtered and not resampled)
    with dask.config.set(scheduler=CustomScheduler(2)):
        scenes_to_save = resample_scene(
            new_scn,
            ["211e"],
            ["grids.conf"],
            None,
            is_polar2grid=True,
            grid_coverage=0.05,
        )
    assert len(scenes_to_save) == 1
    new_scn, data_ids = scenes_to_save[0]
    assert len(new_scn.keys()) == 1  # I01



#!/usr/bin/env python
# encoding: utf-8
# Copyright (C) 2021 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Basic usability tests for the main glue script."""

import contextlib
import os
from glob import glob
from tempfile import gettempdir
from unittest import mock

import dask
import pytest
import yaml
from pytest_lazy_fixtures import lf as lazy_fixture
from satpy.tests.utils import CustomScheduler

from polar2grid.utils.config import get_polar2grid_etc
from polar2grid.utils.warnings import ignore_no_georef


@contextlib.contextmanager
def set_env(**environ):
    """Temporarily set the process environment variables.

    Args:
        environ (dict[str, unicode]): New environment variables to set.

    """
    old_environ = dict(os.environ)
    os.environ.update(environ)
    try:
        yield
    finally:
        os.environ.clear()
        os.environ.update(old_environ)


VIIRS_SDR_FILENAMES = [
    "GDNBO_npp_d20120225_t1801245_e1802487_b01708_c20120226001636568470_noaa_ops.h5",
    "GITCO_npp_d20120225_t1801245_e1802487_b01708_c20120226001734123892_noaa_ops.h5",
    "GMTCO_npp_d20120225_t1801245_e1802487_b01708_c20120226001559910679_noaa_ops.h5",
    "SVDNB_npp_d20120225_t1801245_e1802487_b01708_c20120226002239040837_noaa_ops.h5",
    "SVI01_npp_d20120225_t1801245_e1802487_b01708_c20120226002130255476_noaa_ops.h5",
    "SVI02_npp_d20120225_t1801245_e1802487_b01708_c20120226002313514280_noaa_ops.h5",
    "SVI03_npp_d20120225_t1801245_e1802487_b01708_c20120226002348811096_noaa_ops.h5",
    "SVI04_npp_d20120225_t1801245_e1802487_b01708_c20120226002313755405_noaa_ops.h5",
    "SVI05_npp_d20120225_t1801245_e1802487_b01708_c20120226002349115396_noaa_ops.h5",
    "SVM01_npp_d20120225_t1801245_e1802487_b01708_c20120226002049115233_noaa_ops.h5",
    "SVM02_npp_d20120225_t1801245_e1802487_b01708_c20120226002132385368_noaa_ops.h5",
    "SVM03_npp_d20120225_t1801245_e1802487_b01708_c20120226002053795183_noaa_ops.h5",
    "SVM04_npp_d20120225_t1801245_e1802487_b01708_c20120226002204554472_noaa_ops.h5",
    "SVM05_npp_d20120225_t1801245_e1802487_b01708_c20120226002048565595_noaa_ops.h5",
    "SVM06_npp_d20120225_t1801245_e1802487_b01708_c20120226002138321916_noaa_ops.h5",
    "SVM07_npp_d20120225_t1801245_e1802487_b01708_c20120226002208157737_noaa_ops.h5",
    "SVM08_npp_d20120225_t1801245_e1802487_b01708_c20120226002205860744_noaa_ops.h5",
    "SVM09_npp_d20120225_t1801245_e1802487_b01708_c20120226002205633207_noaa_ops.h5",
    "SVM10_npp_d20120225_t1801245_e1802487_b01708_c20120226002316653329_noaa_ops.h5",
    "SVM11_npp_d20120225_t1801245_e1802487_b01708_c20120226002207165715_noaa_ops.h5",
    "SVM12_npp_d20120225_t1801245_e1802487_b01708_c20120226002134281104_noaa_ops.h5",
    "SVM13_npp_d20120225_t1801245_e1802487_b01708_c20120226002316702845_noaa_ops.h5",
    "SVM14_npp_d20120225_t1801245_e1802487_b01708_c20120226002317383363_noaa_ops.h5",
    "SVM15_npp_d20120225_t1801245_e1802487_b01708_c20120226002348350715_noaa_ops.h5",
    "SVM16_npp_d20120225_t1801245_e1802487_b01708_c20120226002204855751_noaa_ops.h5",
]


@pytest.fixture(scope="session")
def extra_viirs_composite_path(tmp_path_factory):
    tmp_path = tmp_path_factory.mktemp("extra_viirs_composite_path")
    _create_fake_comp(tmp_path)
    _create_fake_comp_enh(tmp_path)
    yield [tmp_path]


def _create_fake_comp(tmp_path):
    from satpy.composites import GenericCompositor

    comps_dict = {
        "composites": {
            "myfakecomp": {
                "compositor": GenericCompositor,
                "prerequisites": [
                    {"name": "I01"},
                    {"name": "I01"},
                    {"name": "I01"},
                ],
                "standard_name": "myfakecomp",
            },
        },
    }

    comp_path = tmp_path / "composites"
    comp_path.mkdir(parents=True)
    viirs_comp_path = comp_path / "viirs.yaml"
    with open(viirs_comp_path, "w") as comp_file:
        yaml.dump(comps_dict, comp_file)


def _create_fake_comp_enh(tmp_path):
    from satpy.enhancements import stretch

    enh_dict = {
        "enhancements": {
            "myfakecomp_default": {
                "standard_name": "myfakecomp",
                "operations": [
                    {
                        "name": "myfakecomp_stretch",
                        "method": stretch,
                        "kwargs": {"stretch": "crude", "min_stretch": 0, "max_stretch": 1},
                    },
                ],
            },
        },
    }

    enh_path = tmp_path / "enhancements"
    enh_path.mkdir(parents=True)
    viirs_enh_file = enh_path / "viirs.yaml"
    with open(viirs_enh_file, "w") as comp_file:
        yaml.dump(enh_dict, comp_file)


@pytest.fixture(scope="session")
def extra_viirs_enhancement_file(tmp_path_factory):
    import yaml
    from satpy.enhancements import stretch

    tmp_path = tmp_path_factory.mktemp("extra_viirs_enhancement_file")

    enh_dict = {
        "enhancements": {
            "myfakeenh_i01": {
                "name": "I01",
                "operations": [
                    {
                        "name": "myfakeenh_i01_stretch",
                        "method": stretch,
                        "kwargs": {"stretch": "crude", "min_stretch": 0, "max_stretch": 1},
                    },
                ],
            },
            "myfakecomp_new_default": {
                "name": "myfakecomp",
                "standard_name": "myfakecomp",
                "operations": [
                    {
                        "name": "myfakecomp_new_stretch",
                        "method": stretch,
                        "kwargs": {"stretch": "crude", "min_stretch": 0, "max_stretch": 1},
                    },
                ],
            },
        },
    }

    enh_path = tmp_path / "enhancements"
    enh_path.mkdir(parents=True)
    viirs_enh_file = enh_path / "blahblahblah.yaml"
    with open(viirs_enh_file, "w") as comp_file:
        yaml.dump(enh_dict, comp_file)
    yield [viirs_enh_file]


@pytest.fixture(scope="session")
def extra_viirs_comp_and_enh(extra_viirs_composite_path, extra_viirs_enhancement_file):
    yield extra_viirs_composite_path + extra_viirs_enhancement_file


@contextlib.contextmanager
def prepare_glue_exec(create_scene_func, max_computes=0, use_polar2grid=True):
    use_str = "1" if use_polar2grid else "0"
    with (
        set_env(USE_POLAR2GRID_DEFAULTS=use_str),
        mock.patch("polar2grid.glue._create_scene") as create_scene,
        dask.config.set(scheduler=CustomScheduler(max_computes)),
    ):
        create_scene.return_value = create_scene_func
        yield


def _create_empty_viirs_sdrs(dst_dir):
    import h5py

    for viirs_fn in VIIRS_SDR_FILENAMES:
        viirs_path = dst_dir / viirs_fn
        h5py.File(viirs_path, "w")


def test_polar2grid_help():
    from polar2grid.glue import main

    with pytest.raises(SystemExit) as e, set_env(USE_POLAR2GRID_DEFAULTS="1"):
        main(["--help"])
    assert e.value.code == 0


def test_geo2grid_help():
    from polar2grid.glue import main

    with pytest.raises(SystemExit) as e, set_env(USE_POLAR2GRID_DEFAULTS="0"):
        main(["--help"])
    assert e.value.code == 0


class TestGlueWithVIIRS:
    def setup_method(self):
        """Wrap HDF5 file handler with our own fake handler."""
        from satpy._config import config_search_paths
        from satpy.readers.viirs_sdr import VIIRSSDRFileHandler
        from satpy.tests.reader_tests.test_viirs_sdr import FakeHDF5FileHandler2

        self.reader_configs = config_search_paths(os.path.join("readers", "viirs_sdr.yaml"))
        # http://stackoverflow.com/questions/12219967/how-to-mock-a-base-class-with-python-mock-library
        self.p = mock.patch.object(VIIRSSDRFileHandler, "__bases__", (FakeHDF5FileHandler2,))
        self.fake_handler = self.p.start()
        self.p.is_local = True

    def teardown_method(self):
        """Stop wrapping the HDF5 file handler."""
        self.p.stop()

    def test_polar2grid_viirs_sdr_list_products(self, tmp_path):
        from polar2grid.glue import main

        _create_empty_viirs_sdrs(tmp_path)
        with set_env(USE_POLAR2GRID_DEFAULTS="1"):
            ret = main(["-r", "viirs_sdr", "-w", "geotiff", "-f", str(tmp_path), "--list-products"])
        assert ret == 0


class TestGlueFakeScene:
    @pytest.mark.parametrize(
        ("scene_fixture", "product_names", "num_outputs", "max_computes"),
        [
            (lazy_fixture("abi_l1b_c01_scene"), ["C01"], 1, 1),
            (lazy_fixture("abi_l1b_airmass_scene"), ["airmass"], 1, 1),
        ],
    )
    def test_abi_scene(self, scene_fixture, product_names, num_outputs, max_computes, chtmpdir):
        from polar2grid.glue import main

        with prepare_glue_exec(scene_fixture, max_computes=max_computes, use_polar2grid=False):
            ret = main(["-r", "abi_l1b", "-w", "geotiff", "-f", str(chtmpdir), "-p"] + product_names)
        output_files = glob(str(chtmpdir / "*.tif"))
        assert len(output_files) == num_outputs
        assert ret == 0

    @pytest.mark.parametrize(
        ("scene_fixture", "product_names", "num_outputs", "extra_flags", "max_computes"),
        [
            # lon/lat persist -> day/night check (I band) -> dynamic grid -> final compute
            (lazy_fixture("viirs_sdr_i01_scene"), [], 1, [], 4),
            # lon/lat persist -> day/night check (I band) -> dynamic grid -> final compute
            (lazy_fixture("viirs_sdr_i01_scene"), [], 1, ["--dnb-saturation-correction"], 4),
            # lon/lat persist -> day/night check (x2 = I band + M band) -> dynamic grid -> final compute
            (lazy_fixture("viirs_sdr_full_scene"), [], 5 + 16 + 3 + 1 + 1 + 1, ["--dnb-saturation-correction"], 5),
            # lon/lat persist -> day/night check (x2 = I band + M band) -> dynamic grid -> final compute
            (
                lazy_fixture("viirs_sdr_full_scene"),
                [],
                5,
                ["-g", "lcc_conus_1km", "--awips-true-color", "--awips-false-color"],
                5,
            ),
            # lon/lat persist -> day/night check (I band) -> final compute
            (lazy_fixture("viirs_sdr_i01_scene"), [], 1, ["--method", "native", "-g", "MAX"], 3),
        ],
    )
    def test_viirs_sdr_scene(self, scene_fixture, product_names, num_outputs, extra_flags, max_computes, chtmpdir):
        from polar2grid.glue import main

        with prepare_glue_exec(scene_fixture, max_computes=max_computes):
            args = ["-r", "viirs_sdr", "-w", "geotiff", "-vvv", "-f", str(chtmpdir)]
            if product_names:
                args.append("-p")
                args.extend(product_names)
            if extra_flags:
                args.extend(extra_flags)
            with ignore_no_georef():
                ret = main(args)
        output_files = glob(str(chtmpdir / "*.tif"))
        assert len(output_files) == num_outputs
        assert ret == 0

    @pytest.mark.parametrize(
        ("extra_config_path", "exp_comps", "exp_enh_names"),
        [
            (lazy_fixture("extra_viirs_composite_path"), ["myfakecomp"], ["myfakecomp_stretch", "gamma"]),
            (lazy_fixture("extra_viirs_enhancement_file"), [], ["myfakeenh_i01_stretch"]),
            (
                lazy_fixture("extra_viirs_comp_and_enh"),
                ["myfakecomp"],
                ["myfakeenh_i01_stretch", "myfakecomp_new_stretch"],
            ),
        ],
    )
    def test_extra_config_path(
        self, extra_config_path, exp_comps, exp_enh_names, viirs_sdr_full_scene, chtmpdir, capsys
    ):
        from polar2grid.glue import main

        args = ["-r", "viirs_sdr", "-w", "geotiff", "-vvv", "-f", str(chtmpdir)]
        for extra_path in extra_config_path:
            args += ["--extra-config-path", str(extra_path)]
        with prepare_glue_exec(viirs_sdr_full_scene):
            ret = main(args + ["--list-products-all"])
        assert ret == 0
        captured = capsys.readouterr()
        for exp_comp in exp_comps:
            assert exp_comp in captured.out

        with prepare_glue_exec(viirs_sdr_full_scene, max_computes=4):
            ret = main(args + ["-p"] + exp_comps + ["i01"])
        assert ret == 0
        captured = capsys.readouterr()
        for exp_enh_name in exp_enh_names:
            assert exp_enh_name in captured.err

        # ensure p2g builtins are processed/added before user custom configs
        p2g_etc = get_polar2grid_etc()
        builtin_path_idx = captured.err.index(f"Adding enhancement configuration from file: {p2g_etc}")  # the first one
        for extra_cpath in extra_config_path:
            if str(extra_cpath).endswith(".yaml"):
                # the YAML was copied to a temp directory, check for that
                extra_cpath = gettempdir()
            path_idx = captured.err.index(f"Adding enhancement configuration from file: {str(extra_cpath)}")
            assert builtin_path_idx < path_idx



#!/usr/bin/env python
# encoding: utf-8
# Copyright (C) 2021 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Test initialization and fixtures."""

from __future__ import annotations

import logging
import os
from pathlib import Path

import pytest

PKG_ROOT = os.path.realpath(os.path.join(os.path.dirname(__file__), ".."))
root_logger = logging.getLogger()

pytest_plugins = [
    "polar2grid.tests._abi_fixtures",
    "polar2grid.tests._viirs_fixtures",
    "polar2grid.tests._avhrr_fixtures",
]


def pytest_configure(config):
    from polar2grid.utils.config import add_polar2grid_config_paths

    add_polar2grid_config_paths()


@pytest.fixture(autouse=True)
def clear_cached_functions():
    from polar2grid.filters._utils import polygon_for_area
    from polar2grid.filters.day_night import _get_sunlight_coverage

    _get_sunlight_coverage.cache_clear()
    polygon_for_area.cache_clear()


@pytest.fixture
def chtmpdir(tmp_path: Path):
    lwd = os.getcwd()
    os.chdir(tmp_path)
    try:
        yield tmp_path
    finally:
        os.chdir(lwd)


@pytest.fixture(scope="session")
def builtin_grids_yaml() -> list[str]:
    return [os.path.join(PKG_ROOT, "grids", "grids.yaml")]


@pytest.fixture(scope="session")
def builtin_test_grids_conf() -> list[str]:
    return [os.path.join(PKG_ROOT, "tests", "etc", "grids.conf")]


@pytest.fixture(autouse=True)
def ensure_logging_framework_not_altered():
    """Protect logger handlers to avoid capsys closing files."""
    before_handlers = list(root_logger.handlers)
    yield
    root_logger.handlers = before_handlers



#!/usr/bin/env python
# encoding: utf-8
# Copyright (C) 2021 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Utilities related to filtering."""

import logging

import dask.array as da
import numpy as np

try:
    # Python 3.9+
    from functools import cache
except ImportError:
    from functools import lru_cache as cache

from typing import Union

from pyresample.boundary import AreaBoundary, AreaDefBoundary, Boundary
from pyresample.geometry import AreaDefinition, SwathDefinition, get_geostationary_bounding_box_in_lonlats
from pyresample.spherical import SphPolygon

logger = logging.getLogger(__name__)

PRGeometry = Union[SwathDefinition, AreaDefinition]


def boundary_for_area(area_def: PRGeometry) -> Boundary:
    """Create Boundary object representing the provided area."""
    if getattr(area_def, "is_geostationary", False):
        adp = Boundary(*get_geostationary_bounding_box_in_lonlats(area_def, nb_points=100))
    else:
        freq_fraction = 0.30 if isinstance(area_def, AreaDefinition) else 0.05
        try:
            adp = area_def.boundary()
            adp.decimate(int(freq_fraction * area_def.shape[0]))
            if adp.contour_poly.area() < 0:
                # https://github.com/ssec/polar2grid/issues/696
                raise ValueError("Failed to generate valid polygon for area. Polygon has a negative area.")
        except ValueError:
            if not isinstance(area_def, SwathDefinition):
                logger.error("Unable to generate bounding geolocation polygon")
                raise
            logger.warning(
                "Geolocation data contains invalid bounding values. Computing entire array to get bounds instead."
            )
            adp = _compute_boundary_from_whole_swath(area_def)
    return adp


def _compute_boundary_from_whole_swath(swath_def: SwathDefinition):
    lons, lats = swath_def.get_lonlats()
    min_lon = np.nanmin(lons)
    max_lon = np.nanmax(lons)
    min_lat = np.nanmin(lats)
    max_lat = np.nanmax(lats)
    min_lon, max_lon, min_lat, max_lat = da.compute(min_lon, max_lon, min_lat, max_lat)
    x_passes_antimeridian = (max_lon - min_lon) > 355
    epsilon = 0.1
    y_is_pole = (max_lat >= 90 - epsilon) or (min_lat <= -90 + epsilon)
    if x_passes_antimeridian and not y_is_pole:
        wrapped_lons = lons % 360
        min_lon, max_lon = da.compute(np.nanmin(wrapped_lons), np.nanmax(wrapped_lons))
        max_lon -= 360
    sides = (
        ([min_lon, max_lon], [max_lat, max_lat]),  # top
        ([max_lon, max_lon], [max_lat, min_lat]),  # right
        ([max_lon, min_lon], [min_lat, min_lat]),  # bot
        ([min_lon, max_lon], [min_lat, max_lat]),  # left
    )
    return AreaBoundary(*sides)


@cache
def polygon_for_area(area_def: PRGeometry) -> SphPolygon:
    boundary = boundary_for_area(area_def)
    return boundary.contour_poly



#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2021 Space Science and Engineering Center (SSEC),
# University of Wisconsin-Madison.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Test grid config helper script."""

from io import StringIO

import pytest
from pyresample import parse_area_file
from pyresample.geometry import AreaDefinition

from polar2grid.grids.config_helper import main as ch_main


def test_help_usage(capsys):
    with pytest.raises(SystemExit) as e:
        ch_main(["--help"])
    assert e.value.code == 0
    sout = capsys.readouterr()
    assert "positional arguments" in sout.out

    # you must provide some parameters
    with pytest.raises(SystemExit) as e:
        ch_main([])
    assert e.value.code != 0
    sout = capsys.readouterr()
    assert "the following arguments are required" in sout.err


@pytest.mark.parametrize(
    ("lon", "lat", "exp"),
    [
        (
            "-95.0",
            "10.0",
            """my_grid:
  projection:
    proj: eqc
    lat_ts: 10
    lat_0: 0
    lon_0: -95
    datum: WGS84
    units: m
    no_defs: null
    type: crs
  shape:
    height: 1500
    width: 1500
  center:
    x: -95.0
    y: 10.0
    units: degrees
  resolution:
    dx: 1000.0
    dy: 1000.0

""",
        ),
        (
            "-95.0",
            "35.0",
            """my_grid:
  projection:
    proj: lcc
    lat_1: 35
    lat_0: 35
    lon_0: -95
    datum: WGS84
    units: m
    no_defs: null
    type: crs
  shape:
    height: 1500
    width: 1500
  center:
    x: -95.0
    y: 35.0
    units: degrees
  resolution:
    dx: 1000.0
    dy: 1000.0

""",
        ),
        (
            "-95.0",
            "80.0",
            """my_grid:
  projection:
    proj: stere
    lat_0: 90
    lat_ts: 80
    lon_0: -95
    datum: WGS84
    units: m
    no_defs: null
    type: crs
  shape:
    height: 1500
    width: 1500
  center:
    x: -95.0
    y: 80.0
    units: degrees
  resolution:
    dx: 1000.0
    dy: 1000.0

""",
        ),
    ],
)
def test_yaml_format(lon, lat, exp, capsys):
    ret = ch_main(["my_grid", lon, lat, "1000", "1000", "1500", "1500"])
    assert ret is None or ret == 0
    sout = capsys.readouterr()
    assert sout.out == exp

    # temp file
    s = StringIO()
    s.write(sout.out)
    s.seek(0)
    areas = parse_area_file([s])
    assert len(areas) == 1
    assert isinstance(areas[0], AreaDefinition)



#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2016-2021 Space Science and Engineering Center (SSEC),
# University of Wisconsin-Madison.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
#     input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
#
# Written by David Hoese    July 2016
# University of Wisconsin-Madison
# Space Science and Engineering Center
# 1225 West Dayton Street
# Madison, WI  53706
# david.hoese@ssec.wisc.edu
"""Script to add coastlines and borders to a geotiff while also creating a PNG."""

from __future__ import annotations

import argparse
import logging
import os
import sys
from pathlib import Path

import numpy as np
import rasterio
from aggdraw import Font
from PIL import Image, ImageFont
from pycoast import ContourWriterAGG
from pydecorate import DecoratorAGG
from pyresample.utils import get_area_def_from_raster
from trollimage.colormap import Colormap

from polar2grid.utils.config import add_polar2grid_config_paths

LOG = logging.getLogger(__name__)
PYCOAST_DIR = os.environ.get("GSHHS_DATA_ROOT")


def _get_rio_colormap(rio_ds, bidx):
    try:
        return rio_ds.colormap(bidx)
    except ValueError:
        return None


def _get_colorbar_vmin_vmax(arg_min, arg_max, metadata, input_dtype):
    scale = metadata.get("scale", metadata.get("scale_factor"))
    offset = metadata.get("offset", metadata.get("add_offset"))
    dtype_min = float(np.iinfo(input_dtype).min)
    dtype_max = float(np.iinfo(input_dtype).max)
    if arg_min is None and scale is None:
        LOG.warning(
            "Colorbar min/max metadata not found and not provided "
            "on the command line. Defaulting to data type limits."
        )
        return dtype_min, dtype_max

    if arg_min is not None:
        vmin = float(arg_min)
        vmax = float(arg_max)
    else:
        scale, offset = _convert_scale_offset_to_float(scale, offset)
        vmin, vmax = _vmin_vmax_from_scale_offset(scale, offset, dtype_min, dtype_max)
    return vmin, vmax


def _convert_scale_offset_to_float(scale: str, offset: str) -> tuple[float, float]:
    scale = float(scale)
    offset = float(offset)
    if np.isnan(scale) or np.isnan(offset):
        raise ValueError(
            "Can't automatically set colorbar limits with "
            "geotiff metadata as scale/offset are set to "
            "NaN. This indicates a non-linear enhancement or "
            "RGB/A composite that was enhanced. These cases "
            "cannot be represented properly by a colorbar."
        )
    return scale, offset


def _vmin_vmax_from_scale_offset(scale: float, offset: float, dtype_min: int, dtype_max: int) -> tuple[float, float]:
    delta = dtype_max - dtype_min
    vmin = offset
    vmax = delta * scale + offset
    # floating point error made it not an integer
    if abs(vmin - np.round(vmin, 0)) <= 0.001:
        vmin = np.round(vmin, 0)
    if abs(vmax - np.round(vmax, 0)) <= 0.001:
        vmax = np.round(vmax, 0)
    return vmin, vmax


def _apply_decorator_alignment(dc, align):
    if align == "top":
        dc.align_top()
        dc.write_horizontally()
    elif align == "bottom":
        dc.align_bottom()
        dc.write_horizontally()
    elif align == "left":
        dc.align_left()
        dc.write_vertically()
    elif align == "right":
        dc.align_right()
        dc.write_vertically()


def _add_colorbar_to_image(img, font, tick_color, align, **kwargs):
    dc = DecoratorAGG(img)
    _apply_decorator_alignment(dc, align)
    dc.add_scale(
        font=font,
        line=tick_color,
        **kwargs,
    )


def get_parser():
    parser = argparse.ArgumentParser(
        description="Add overlays to a GeoTIFF file and save as a PNG file.",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    _add_coastlines_arguments(parser)
    _add_rivers_arguments(parser)
    _add_grid_arguments(parser)
    _add_borders_arguments(parser)
    _add_colorbar_arguments(parser)
    _add_global_arguments(parser)
    parser.add_argument("input_tiff", nargs="+", help="Input geotiff(s) to process")
    return parser


def _add_coastlines_arguments(parser: argparse.ArgumentParser) -> None:
    group = parser.add_argument_group("coastlines")
    group.add_argument("--add-coastlines", action="store_true", help="Add coastlines")
    group.add_argument(
        "--coastlines-resolution",
        choices="clihf",
        default="i",
        help="Resolution of coastlines to add (crude, low, intermediate, high, full)",
    )
    group.add_argument(
        "--coastlines-level",
        choices=range(1, 7),
        type=int,
        default=4,
        help="Level of detail from the selected resolution dataset",
    )
    group.add_argument(
        "--coastlines-outline",
        default=["yellow"],
        nargs="*",
        help="Color of coastline lines (color name or 3 RGB integers)",
    )
    group.add_argument("--coastlines-fill", default=None, nargs="*", help="Color of land")
    group.add_argument("--coastlines-width", default=1.0, type=float, help="Width of coastline lines")


def _add_rivers_arguments(parser: argparse.ArgumentParser) -> None:
    group = parser.add_argument_group("rivers")
    group.add_argument("--add-rivers", action="store_true", help="Add rivers grid")
    group.add_argument(
        "--rivers-resolution",
        choices="clihf",
        default="c",
        help="Resolution of rivers to add (crude, low, intermediate, high, full)",
    )
    group.add_argument(
        "--rivers-level", choices=range(0, 11), type=int, default=5, help="Level of detail for river lines"
    )
    group.add_argument(
        "--rivers-outline", default=["blue"], nargs="*", help="Color of river lines (color name or 3 RGB integers)"
    )
    group.add_argument("--rivers-width", default=1.0, type=float, help="Width of rivers lines")


def _add_grid_arguments(parser: argparse.ArgumentParser) -> None:
    group = parser.add_argument_group("grid")
    group.add_argument("--add-grid", action="store_true", help="Add lat/lon grid")
    group.add_argument("--grid-no-text", dest="grid_text", action="store_false", help="Add labels to lat/lon grid")
    group.add_argument("--grid-text-size", default=32, type=int, help="Lat/lon grid text font size")
    group.add_argument("--grid-font", default="Vera.ttf", help="Path to TTF font (package provided or custom path)")
    group.add_argument(
        "--grid-fill", nargs="*", default=["cyan"], help="Color of grid text (color name or 3 RGB integers)"
    )
    group.add_argument(
        "--grid-outline", nargs="*", default=["cyan"], help="Color of grid lines (color name or 3 RGB integers)"
    )
    group.add_argument(
        "--grid-minor-outline", nargs="*", default=["cyan"], help="Color of tick lines (color name or 3 RGB integers)"
    )
    group.add_argument(
        "--grid-D", nargs=2, default=(10.0, 10.0), type=float, help="Degrees between grid lines (lon, lat)"
    )
    group.add_argument(
        "--grid-d", nargs=2, default=(2.0, 2.0), type=float, help="Degrees between tick lines (lon, lat)"
    )
    group.add_argument(
        "--grid-lon-placement", choices=["tl", "lr", "lc", "cc"], default="tb", help="Longitude label placement"
    )
    group.add_argument(
        "--grid-lat-placement", choices=["tl", "lr", "lc", "cc"], default="lr", help="Latitude label placement"
    )
    group.add_argument("--grid-width", default=1.0, type=float, help="Width of grid lines")


def _add_borders_arguments(parser: argparse.ArgumentParser) -> None:
    group = parser.add_argument_group("borders")
    group.add_argument("--add-borders", action="store_true", help="Add country and/or region borders")
    group.add_argument(
        "--borders-resolution",
        choices="clihf",
        default="i",
        help="Resolution of borders to add (crude, low, intermediate, high, full)",
    )
    group.add_argument(
        "--borders-level", choices=range(1, 4), default=2, type=int, help="Level of detail for border lines"
    )
    group.add_argument(
        "--borders-outline", default=["white"], nargs="*", help="Color of border lines (color name or 3 RGB integers)"
    )
    group.add_argument("--borders-width", default=1.0, type=float, help="Width of border lines")


def _add_colorbar_arguments(parser: argparse.ArgumentParser) -> None:
    group = parser.add_argument_group("colorbar")
    group.add_argument("--add-colorbar", action="store_true", help="Add colorbar on top of image")
    group.add_argument(
        "--colorbar-colormap-file",
        help=argparse.SUPPRESS,
        # help="Specify the colormap file that was used to "
        # "colorize the provided RGB geotiff. Only used if "
        # "the provided geotiff is RGB/A. Otherwise the "
        # "geotiff is expected to include the colormap as "
        # "a geotiff color table.",
    )
    group.add_argument("--colorbar-width", type=int, help="Number of pixels wide")
    group.add_argument("--colorbar-height", type=int, help="Number of pixels high")
    group.add_argument(
        "--colorbar-extend", action="store_true", help="Extend colorbar to full width/height of the image"
    )
    group.add_argument(
        "--colorbar-tick-marks", type=float, default=5.0, help="Major tick and tick label interval in data units"
    )
    group.add_argument("--colorbar-minor-tick-marks", type=float, default=1.0, help="Minor tick interval in data units")
    group.add_argument("--colorbar-text-size", default=32, type=int, help="Tick label font size")
    group.add_argument(
        "--colorbar-text-color", nargs="*", default=["black"], help="Color of tick text (color name or 3 RGB integers)"
    )
    group.add_argument("--colorbar-font", default="Vera.ttf", help="Path to TTF font (package provided or custom path)")
    group.add_argument(
        "--colorbar-align",
        choices=["left", "top", "right", "bottom"],
        default="bottom",
        help="Which side of the image to place the colorbar",
    )
    group.add_argument("--colorbar-vertical", action="store_true", help="DEPRECATED")
    group.add_argument(
        "--colorbar-min",
        type=float,
        help="Minimum data value of the colorbar."
        " Defaults to 'min_in' of input metadata or"
        " minimum value of the data otherwise.",
    )
    group.add_argument(
        "--colorbar-max",
        type=float,
        help="Maximum data value of the colorbar."
        " Defaults to 'max_in' of input metadata or"
        " maximum value of the data otherwise.",
    )
    group.add_argument("--colorbar-units", help="Units marker to include in the colorbar text")
    group.add_argument("--colorbar-title", help="Title shown with the colorbar")


def _add_global_arguments(parser: argparse.ArgumentParser) -> None:
    parser.add_argument(
        "--shapes-dir",
        default=PYCOAST_DIR,
        help="Specify alternative directory for coastline shape files (default: GSHSS_DATA_ROOT)",
    )
    parser.add_argument(
        "--cache-dir",
        default=None,
        help="Specify directory where cached coastline output can be stored and accessed in later "
        "executions. The cache will never be cleared by this script. Caching depends on the grid "
        "of the image and the decorations added to the image.",
    )
    parser.add_argument(
        "--cache-regenerate",
        action="store_true",
        help="Force regeneration of any cached overlays. Requires '--cache-dir'.",
    )
    parser.add_argument(
        "-o",
        "--output",
        dest="output_filename",
        nargs="+",
        help="Specify the output filename (default replace '.tif' with '.png')",
    )
    parser.add_argument(
        "-v",
        "--verbose",
        dest="verbosity",
        action="count",
        default=0,
        help="each occurrence increases verbosity 1 level through ERROR-WARNING-INFO-DEBUG (default INFO)",
    )


def main(argv=sys.argv[1:]):
    parser = get_parser()
    args = parser.parse_args(argv)

    levels = [logging.ERROR, logging.WARN, logging.INFO, logging.DEBUG]
    logging.basicConfig(level=levels[min(3, args.verbosity)])
    add_polar2grid_config_paths()

    if args.output_filename is None:
        args.output_filename = [x[:-3] + "png" for x in args.input_tiff]
    elif len(args.output_filename) != len(args.input_tiff):
        LOG.error("Output filenames must be equal to number of input tiffs")
        return -1

    if not (args.add_borders or args.add_coastlines or args.add_grid or args.add_rivers or args.add_colorbar):
        LOG.error("Please specify one of the '--add-X' options to modify the image")
        return -1

    if args.cache_dir and not os.path.isdir(args.cache_dir):
        LOG.info(f"Creating cache directory: {args.cache_dir}")
        os.makedirs(args.cache_dir, exist_ok=True)

    # we may be dealing with large images that look like decompression bombs
    # let's turn off the check for the image size in PIL/Pillow
    Image.MAX_IMAGE_PIXELS = None
    # gather all options into a single dictionary that we can pass to pycoast
    pycoast_options = _args_to_pycoast_dict(args)
    colorbar_kwargs = _args_to_colorbar_kwargs(args) if args.add_colorbar else {}
    for input_tiff, output_filename in zip(args.input_tiff, args.output_filename, strict=True):
        _process_one_image(input_tiff, output_filename, pycoast_options, args.shapes_dir, colorbar_kwargs)
    return 0


def _args_to_pycoast_dict(args):
    opts = {}
    if args.add_coastlines:
        opts["coasts"] = _args_to_coastlines_dict(args)
    if args.add_rivers:
        opts["rivers"] = _args_to_rivers_dict(args)
    if args.add_borders:
        opts["borders"] = _args_to_borders_dict(args)
    if args.add_grid:
        opts["grid"] = _args_to_grid_dict(args)
    if args.cache_dir:
        opts["cache"] = {
            # add "add_coastlines" prefix to cached image name
            "file": os.path.join(args.cache_dir, "add_coastlines"),
            "regenerate": args.cache_regenerate,
        }

    return opts


def _args_to_coastlines_dict(args):
    outline = (
        args.coastlines_outline[0]
        if len(args.coastlines_outline) == 1
        else tuple(int(x) for x in args.coastlines_outline)
    )
    if args.coastlines_fill:
        fill = (
            args.coastlines_fill[0] if len(args.coastlines_fill) == 1 else tuple(int(x) for x in args.coastlines_fill)
        )
    else:
        fill = None
    coasts_dict = {
        "resolution": args.coastlines_resolution,
        "level": args.coastlines_level,
        "width": args.coastlines_width,
        "outline": outline,
        "fill": fill,
    }
    return coasts_dict


def _args_to_rivers_dict(args):
    outline = args.rivers_outline[0] if len(args.rivers_outline) == 1 else tuple(int(x) for x in args.rivers_outline)
    rivers_dict = {
        "resolution": args.rivers_resolution,
        "level": args.rivers_level,
        "width": args.rivers_width,
        "outline": outline,
    }
    return rivers_dict


def _args_to_borders_dict(args):
    outline = args.borders_outline[0] if len(args.borders_outline) == 1 else tuple(int(x) for x in args.borders_outline)
    borders_dict = {
        "resolution": args.borders_resolution,
        "level": args.borders_level,
        "width": args.borders_width,
        "outline": outline,
    }
    return borders_dict


def _args_to_grid_dict(args):
    outline = args.grid_outline[0] if len(args.grid_outline) == 1 else tuple(int(x) for x in args.grid_outline)
    minor_outline = (
        args.grid_minor_outline[0]
        if len(args.grid_minor_outline) == 1
        else tuple(int(x) for x in args.grid_minor_outline)
    )
    fill = args.grid_fill[0] if len(args.grid_fill) == 1 else tuple(int(x) for x in args.grid_fill)
    font_path = find_font(args.grid_font, args.grid_text_size)
    font = Font(outline, font_path, size=args.grid_text_size)
    grid_dict = {
        "lon_major": args.grid_D[0],
        "lat_major": args.grid_D[1],
        "lon_minor": args.grid_d[0],
        "lat_minor": args.grid_d[1],
        "font": font,
        "fill": fill,
        "outline": outline,
        "minor_outline": minor_outline,
        "write_text": args.grid_text,
        "width": args.grid_width,
        "lon_placement": args.grid_lon_placement,
        "lat_placement": args.grid_lat_placement,
    }
    return grid_dict


def _args_to_colorbar_kwargs(args):
    font_color = args.colorbar_text_color
    font_color = font_color[0] if len(font_color) == 1 else tuple(int(x) for x in font_color)
    font_path = find_font(args.colorbar_font, args.colorbar_text_size)
    # this actually needs an aggdraw font
    font = Font(font_color, font_path, size=args.colorbar_text_size)

    if args.colorbar_width is None or args.colorbar_height is None:
        if args.colorbar_width is not None or args.colorbar_height is not None:
            LOG.warning(
                "'--colorbar-width' and '--colorbar-height' were not both specified. " "Forcing '--colorbar-extend'."
            )
        args.colorbar_extend = True

    colorbar_kwargs = {
        "font": font,
        "tick_color": font_color,
        "cmin": args.colorbar_min,
        "cmax": args.colorbar_max,
        "align": args.colorbar_align,
        "extend": args.colorbar_extend,
        "tick_marks": args.colorbar_tick_marks,
        "minor_tick_marks": args.colorbar_minor_tick_marks,
        "title": args.colorbar_title,
        "unit": args.colorbar_units,
    }
    if args.colorbar_width:
        colorbar_kwargs["width"] = args.colorbar_width
    if args.colorbar_height:
        colorbar_kwargs["height"] = args.colorbar_height
    return colorbar_kwargs


def find_font(font_name, size):
    try:
        font = ImageFont.truetype(font_name, size)
        return font.path
    except IOError as err:
        font_path = Path(__file__).parent / "fonts" / font_name
        if not font_path.is_file():
            raise ValueError(f"Font path does not exist: {font_path}") from err
        return str(font_path)


def _process_one_image(
    input_tiff: str, output_filename: str, pycoast_options: dict, shapes_dir: str, colorbar_kwargs: dict
) -> None:
    LOG.info("Creating {} from {}".format(output_filename, input_tiff))
    img = Image.open(input_tiff)
    all_tiff_tags = img.tag_v2
    img_bands = img.getbands()
    num_bands = len(img_bands)
    # P = palette which we assume to be an RGBA colormap
    img = img.convert("RGBA" if num_bands in (2, 4) or "P" in img_bands else "RGB")
    if pycoast_options:
        area_id = os.path.splitext(input_tiff[0])[0]
        area_def = get_area_def_from_raster(input_tiff, area_id=area_id)
        cw = ContourWriterAGG(shapes_dir)
        cw.add_overlay_from_dict(pycoast_options, area_def, background=img)

    if colorbar_kwargs:
        colorbar_kwargs = colorbar_kwargs.copy()
        cmin = colorbar_kwargs.pop("cmin")
        cmax = colorbar_kwargs.pop("cmax")
        cmap = _get_colormap_object(input_tiff, num_bands, cmin, cmax)
        _add_colorbar_to_image(img, colormap=cmap, **colorbar_kwargs)

    kwargs = {}
    if output_filename.endswith(".tif") or output_filename.endswith(".tiff"):
        # Copy geotiff/tiff tags if output is TIFF
        geotiff_tags = {tag_num: tag_val for tag_num, tag_val in all_tiff_tags.items() if tag_num > 30000}
        kwargs = {"tiffinfo": geotiff_tags}
    img.save(output_filename, **kwargs)


def _get_colormap_object(input_tiff, num_bands, cmin, cmax):
    rio_ds = rasterio.open(input_tiff)
    input_dtype = np.dtype(rio_ds.meta["dtype"])
    colormap_csv = rio_ds.tags().get("colormap")
    rio_ct = _get_rio_colormap(rio_ds, 1)
    metadata = rio_ds.tags()
    cmap = _convert_table_to_cmap_or_default_bw(input_dtype, rio_ct, num_bands)
    if num_bands in (3, 4) and colormap_csv is None:
        raise ValueError("RGB and RGBA geotiffs must have a colormap " "specified with '--colorbar-colormap-file'.")
    if num_bands in (3, 4) or colormap_csv is not None:
        cmap = Colormap.from_string(colormap_csv)
    vmin, vmax = _get_colorbar_vmin_vmax(cmin, cmax, metadata, input_dtype)
    cmap = cmap.set_range(vmin, vmax, inplace=False)
    return cmap


def _convert_table_to_cmap_or_default_bw(band_dtype, band_ct, band_count):
    max_val = np.iinfo(band_dtype).max
    # if we have an alpha band then include the entire colormap
    # otherwise assume it is using 0 as a fill value
    start_idx = 1 if band_count == 1 else 0
    if band_ct is None:
        # all black colormap
        # don't assume anything about the colors or scaling of the image if the
        # user didn't provide enough information
        color_iter = ((idx / float(max_val), (0.0, 0.0, 0.0, 1.0)) for idx in range(max_val))
        color_iter = list(color_iter)
    else:
        color_iter = ((idx / float(max_val), color) for idx, color in sorted(band_ct.items())[start_idx:])
        color_iter = ((idx, tuple(x / float(max_val) for x in color)) for idx, color in color_iter)
    cmap = Colormap(*color_iter)
    return cmap


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Script manually called to generate the summary table in the documentation.

The Polar2Grid summary table contains the possible input and output, reader
and writer combinations.

Example output:

    .. list-table::
        :header-rows: 1

        * - Input Source
          - Example Filename Pattern
          - Output Type
          - Polar2Grid Execution
        * - 1
          - 2
          - 3
          - 4

"""

import os
import sys
from collections import namedtuple

script_dir = os.path.realpath(os.path.dirname(__file__))
output_pathname = os.path.join(script_dir, "summary_table.rst")

COLUMN_ORDER = ["source", "input_patterns", "output_type", "reader", "writer", "legacy_script"]
COLUMN_TITLE = {
    "source": "Input Source",
    "input_patterns": "Input Filename Pattern",
    "output_type": "Output Type",
    "reader": "Reader Name",
    "writer": "Writer Name",
    "legacy_script": "Legacy Script",
}


class TableRow(
    namedtuple("TableRow", ("source", "input_patterns", "output_type", "reader", "writer", "legacy_script"))
):
    def __new__(cls, *args):
        if len(args) < 6:
            args = args + ("{reader}2{writer}.sh".format(reader=args[3], writer=args[4]),)
        return super(TableRow, cls).__new__(cls, *args)


summary_table = [
    TableRow(
        "Suomi-NPP VIIRS Sensor Data Records",
        ["SVI01_npp_*.h5", "GITCO_npp_*.h5"],
        "8-bit single band GeoTIFF",
        "viirs_sdr",
        "gtiff",
        "viirs2gtiff.sh",
    ),
    TableRow(
        '"',
        '"',
        "AWIPS NetCDF3",
        "viirs_sdr",
        "awips",
        "viirs2awips.sh",
    ),
    TableRow(
        '"',
        '"',
        "HDF5",
        "viirs_sdr",
        "hdf5",
        "viirs2hdf5.sh",
    ),
    TableRow(
        '"',
        '"',
        "Binary",
        "viirs_sdr",
        "binary",
        "viirs2binary.sh",
    ),
    TableRow(
        '"',
        '"',
        "24-bit true and false color GeoTIFF",
        "crefl",
        "gtiff",
    ),
    TableRow(
        "Aqua and Terra MODIS Level 1b (IMAPP or NASA archive files)",
        ["MOD021KM*.hdf", "MOD03*.hdf", "", "or", "", "t1.*1000m.hdf", "t1.*.geo.hdf"],
        "8 bit single band GeoTIFF",
        "modis",
        "gtiff",
    ),
    TableRow(
        '"',
        '"',
        "AWIPS NetCDF3",
        "modis",
        "awips",
    ),
    TableRow(
        '"',
        '"',
        "HDF5",
        "modis",
        "hdf5",
    ),
    TableRow(
        '"',
        '"',
        "Binary",
        "modis",
        "binary",
    ),
    TableRow(
        '"',
        '"',
        "24-bit true and false color GeoTIFF",
        "crefl",
        "gtiff",
    ),
    TableRow(
        "NOAA-18, NOAA-19, Metop-A and Metop-B AVHRR AAPP Level 1b",
        ["hrpt_noaa18_*.l1b"],
        "8 bit single band GeoTIFF",
        "avhrr",
        "gtiff",
    ),
    TableRow(
        '"',
        '"',
        "AWIPS NetCDF3",
        "avhrr",
        "awips",
    ),
    TableRow(
        '"',
        '"',
        "HDF5",
        "avhrr",
        "hdf5",
    ),
    TableRow(
        '"',
        '"',
        "Binary",
        "avhrr",
        "binary",
    ),
    TableRow(
        "GCOM-W1 ASMR2 L1B",
        ["GW1AM2*L1DLBTBR*.h5"],
        "8 bit single band GeoTIFF",
        "amsr2_l1b",
        "gtiff",
        "N/A",
    ),
    TableRow(
        '"',
        '"',
        "AWIPS NetCDF3",
        "amsr2_l1b",
        "awips",
        "N/A",
    ),
    TableRow(
        '"',
        '"',
        "HDF5",
        "amsr2_l1b",
        "hdf5",
        "N/A",
    ),
    TableRow(
        '"',
        '"',
        "Binary",
        "amsr2_l1b",
        "binary",
        "N/A",
    ),
]


def main():
    import argparse

    parser = argparse.ArgumentParser()
    parser.parse_args()

    out_file = open(output_pathname, "w")

    s = """.. File auto-generated by ``generate_summary_table.py``

.. list-table:: Reader/Writer Summary Table
    :header-rows: 1

"""

    header_row = "    * - " + "\n      - ".join([COLUMN_TITLE[n] for n in COLUMN_ORDER]) + "\n"
    s += header_row

    for entry in summary_table:
        items = []
        for n in COLUMN_ORDER:
            item = getattr(entry, n)
            if isinstance(item, (list, tuple)):
                item = "\n        ".join(item)
            items.append(item)
        s += "    * - " + "\n      - ".join(items) + "\n"

    out_file.write(s)


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2021 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Base class and other utilities for wrapping Satpy readers."""

from __future__ import annotations

import logging
from functools import cached_property
from typing import Optional, Union

import satpy
from satpy import DataID, DataQuery, Scene

from polar2grid.utils.dynamic_imports import get_reader_attr
from polar2grid.utils.legacy_compat import AliasHandler

logger = logging.getLogger(__name__)


class ReaderProxyBase:
    """Helper to provide Polar2Grid-specific information about a reader.

    In the most common cases, a subclass should define the following:

    * **is_polar2grid_reader** or **is_geo2grid_reader**: If this reader
      should be used for Polar2Grid or Geo2Grid.
    * **get_default_products**: Get a list of Polar2Grid names (not Satpy
      names or DataQuery objects) that should be loaded if the user hasn't
      specified any.
    * **get_all_products**: Get all Polar2Grid names that this reader could
      possibly load. This list of products will be used by other methods to
      determine what available products for loading are custom or Satpy
      products and what products are "guaranteed" products from
      Polar2Grid/Geo2Grid.
    * **_aliases**: A property that returns a dictionary mapping Polar2Grid
      name for a product to a Satpy DataQuery or name.

    """

    is_polar2grid_reader: bool = False
    is_geo2grid_reader: bool = False

    def __init__(self, scn: Scene, user_products: list[str]):
        self.scn = scn
        self._orig_user_products = user_products

    @cached_property
    def _alias_handler(self):
        return self._create_alias_handler()

    @classmethod
    def from_reader_name(cls, reader_name: str, *args):
        reader_info_cls = get_reader_attr(reader_name, "ReaderProxy")
        if reader_info_cls is None:
            reader_info_cls = cls
        return reader_info_cls(*args)

    @property
    def _binary_name(self):
        if self.is_geo2grid_reader:
            return "geo2grid"
        return "polar2grid"

    @property
    def _aliases(self) -> dict[str, Union[DataQuery, str]]:
        return {}

    def get_default_products(self) -> list[str]:
        """Get products to load if user hasn't specified any others."""
        return []

    def get_all_products(self) -> list[str]:
        """Get all polar2grid products that could be loaded."""
        return []

    def get_available_products(
        self,
        p2g_product_names: Optional[list[str]] = None,
        possible_satpy_ids: Optional[list[DataID]] = None,
    ) -> tuple[list[str], list[str]]:
        """Get custom/satpy products and polar2grid products that are available for loading."""
        if possible_satpy_ids is None:
            possible_satpy_ids = self.scn.available_dataset_ids(composites=True)
        available_custom_products = self.get_user_custom_products()
        if p2g_product_names is None:
            p2g_product_names = self.get_all_products()
            if not p2g_product_names:
                logger.warning(
                    "Provided readers are not configured in %s. All "
                    "products will be listed with internal Satpy names.",
                    self._binary_name,
                )
                return sorted(set([x["name"] for x in possible_satpy_ids])), []
        return self._alias_handler.available_product_names(
            p2g_product_names, available_custom_products, possible_satpy_ids
        )

    def get_user_custom_products(
        self,
    ):
        satpy_and_p2g_ids = self.scn.available_dataset_ids(composites=True)
        with satpy.config.set(config_path=[]):
            satpy_only_ids = self.scn.available_dataset_ids(composites=True)
        return sorted(set(satpy_and_p2g_ids) - set(satpy_only_ids))

    def apply_p2g_name_to_scene(
        self,
        scn: Scene,
    ):
        """Add a 'p2g_name' attribute to each DataArray in the provided Scene."""
        self._alias_handler.apply_p2g_name_to_scene(scn)

    def get_satpy_products_to_load(self) -> Optional[list[Union[DataQuery, str]]]:
        """Get Satpy product names and DataQuery objects to load."""
        no_handler = self._alias_handler is None
        no_user_products = not self._orig_user_products
        no_defaults = not self.get_default_products()
        if no_handler and no_user_products and no_defaults:
            logger.error(
                "Reader does not have a default set of products to load, "
                "please specify products to load with `--products`."
            )
            return None
        elif no_handler:
            # This shouldn't happen unless _create_alias_handler is changed
            raise RuntimeError("Converting to Satpy identifiers failed in an unexpected way.")

        products_to_load = list(self._alias_handler.convert_p2g_name_to_satpy())
        if not products_to_load and no_user_products and not no_defaults:
            msg = "Default products were not found in available file products."
            debug_msg = "Default products were not found in available file products:\n\t{}"
            debug_msg = debug_msg.format("\n\t".join(self.scn.all_dataset_names()))
            logger.error(msg)
            logger.debug(debug_msg)

        return products_to_load

    def _create_alias_handler(self, allow_empty_list=False):
        user_products = self._orig_user_products
        default_products = self.get_default_products()
        all_dataset_names = None
        use_defaults = not user_products and default_products
        if use_defaults:
            logger.info("Using default product list: {}".format(default_products))
            user_products = default_products
            all_dataset_names = self.scn.all_dataset_names(composites=True)
        elif not user_products and not allow_empty_list:
            return None

        alias_handler = AliasHandler(self._aliases, user_products)
        if all_dataset_names is not None:
            # only use defaults that actually exist for the provided files
            alias_handler.remove_unknown_user_products(all_dataset_names)
        return alias_handler



#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2021 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""The MODIS L2 Reader operates on HDF4 Level 2 files from the MODIS instrument.

These files are created from Level 1b data from the Moderate Resolution
Imaging Spectroradiometer (MODIS) instruments on the Aqua and Terra
satellites. The reader is designed to work with files created by the IMAPP
direct broadcast processing system (file naming conventions such as
a1.17006.1855.mod35.hdf), but can support other types of L2 files, including
the NASA archived files (file naming conventions such as
MOD35_L2.A2017004.1732.005.2017023210017.hdf).  The
reader can be specified to the ``polar2grid.sh`` script by using the reader
name ``modis_l2``.

It provides the following products:

    +--------------------+--------------------------------------------+
    | Product Name       | Description                                |
    +====================+============================================+
    | cloud_mask         | Cloud Mask                                 |
    +--------------------+--------------------------------------------+
    | land_sea_mask      | Land Sea Mask                              |
    +--------------------+--------------------------------------------+
    | snow_ice_mask      | Snow Ice Mask                              |
    +--------------------+--------------------------------------------+
    | sst                | Sea Surface Temperature                    |
    +--------------------+--------------------------------------------+
    | lst                | Land Surface Temperature                   |
    +--------------------+--------------------------------------------+
    | ndvi               | Normalized Difference Vegetation Index     |
    +--------------------+--------------------------------------------+
    | ist                | Ice Surface Temperature                    |
    +--------------------+--------------------------------------------+
    | inversion_strength | Inversion Strength                         |
    +--------------------+--------------------------------------------+
    | inversion_depth    | Inversion Depth                            |
    +--------------------+--------------------------------------------+
    | ice_concentration  | Ice Concentration                          |
    +--------------------+--------------------------------------------+
    | ctt                | Cloud Top Temperature                      |
    +--------------------+--------------------------------------------+
    | tpw                | Total Precipitable Water                   |
    +--------------------+--------------------------------------------+

"""

from __future__ import annotations

from argparse import ArgumentParser, _ArgumentGroup
from typing import Optional

from satpy import DataQuery

from ._base import ReaderProxyBase

PREFERRED_CHUNK_SIZE: int = 1354 * 2  # roughly the number columns in a 500m dataset

PRODUCTS = [
    "cloud_mask",
    "land_sea_mask",
    "snow_ice_mask",
    "sst",
    "lst",
    "ndvi",
    "ist",
    "inversion_strength",
    "inversion_depth",
    "ice_concentration",
    "ctt",
    "tpw",
]

PRODUCT_ALIASES = {
    "ist": DataQuery(name="ice_surface_temperature"),
    "sst": DataQuery(name="sea_surface_temperature"),
    "ctt": DataQuery(name="cloud_top_temperature"),
    "tpw": DataQuery(name="water_vapor"),
}


class ReaderProxy(ReaderProxyBase):
    """Provide Polar2Grid-specific information about this reader's products."""

    is_polar2grid_reader = True

    def get_default_products(self) -> list[str]:
        """Get products to load if users hasn't specified any others."""
        return PRODUCTS

    def get_all_products(self) -> list[str]:
        """Get all polar2grid products that could be loaded."""
        return PRODUCTS

    @property
    def _aliases(self) -> dict[str, DataQuery]:
        return PRODUCT_ALIASES


def add_reader_argument_groups(
    parser: ArgumentParser, group: Optional[_ArgumentGroup] = None
) -> tuple[Optional[_ArgumentGroup], Optional[_ArgumentGroup]]:
    """Add reader-specific command line arguments to an existing argument parser.

    If ``group`` is provided then arguments are added to this group. If not,
    a new group is added to the parser and arguments added to this new group.

    """
    if group is None:
        group = parser.add_argument_group(title="MODIS L1B Reader")
    return group, None



#!/usr/bin/env python
# encoding: utf-8
# Copyright (C) 2021 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Tests for legacy handling utilities."""

from __future__ import annotations

import logging

import pytest

from polar2grid.utils.legacy_compat import convert_p2g_pattern_to_satpy


@pytest.mark.parametrize(
    ("input_str", "exp_str", "num_exp_warnings"),
    [
        ("{p2g_name}_{start_time:%Y%m%d_%H%M%S}.tif", "{p2g_name}_{start_time:%Y%m%d_%H%M%S}.tif", 0),
        (
            "{satellite}_{instrument}_{product_name}_{begin_time}.tif",
            "{platform_name}_{sensor}_{p2g_name}_{start_time:%Y%m%d_%H%M%S}.tif",
            4,
        ),
        (
            "{grid_name}_{product_name}_{begin_time_HHMM}_{end_time_HHMM}.tif",
            "{area.area_id}_{p2g_name}_{start_time:%H%M}_{end_time:%H%M}.tif",
            4,
        ),
    ],
)
def test_pattern_formatting(input_str, exp_str, num_exp_warnings, caplog):
    with caplog.at_level(logging.WARNING):
        new_str = convert_p2g_pattern_to_satpy(input_str)
    assert new_str == exp_str
    legacy_warns = [record for record in caplog.records if "to avoid this warning" in record.message]
    assert len(legacy_warns) == num_exp_warnings



#!/usr/bin/env python
# encoding: utf-8
# Copyright (C) 2021 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Basic usability tests for the add_colormap script."""

from __future__ import annotations

import os

import pytest
import rasterio
import satpy

from polar2grid.tests import TEST_ETC_DIR

from .test_add_coastlines import _create_fake_l_geotiff


def test_add_colormap_help():
    from polar2grid.add_colormap import main

    with pytest.raises(SystemExit) as e:
        main(["--help"])
    assert e.value.code == 0


@pytest.mark.parametrize(
    ("cmap_path", "exp_first_color", "exp_middle_color", "exp_last_color"),
    [
        (
            os.path.join(TEST_ETC_DIR, "colormaps", "amsr2_36h.cmap"),
            (0, 0, 0, 255),
            (128, 255, 130, 255),
            (128, 0, 0, 255),
        ),
        (os.path.join("colormaps", "amsr2_36h.cmap"), (0, 0, 0, 255), (128, 255, 130, 255), (128, 0, 0, 255)),
        (
            os.path.join(TEST_ETC_DIR, "colormaps", "WV_Chile_Short.cmap"),
            (0, 0, 0, 255),
            (0, 47, 144, 255),
            (127, 127, 127, 255),
        ),
        (os.path.join(TEST_ETC_DIR, "colormaps", "reds.cmap"), (0, 0, 0, 255), (129, 3, 39, 255), (255, 0, 0, 255)),
    ],
)
def test_add_colormap_basic_l(tmp_path, cmap_path, exp_first_color, exp_middle_color, exp_last_color):
    from polar2grid.add_colormap import main

    fp = str(tmp_path / "test.tif")
    _create_fake_l_geotiff(fp)
    with satpy.config.set(config_path=[TEST_ETC_DIR]):
        ret = main([cmap_path, fp])
    assert ret in [None, 0]

    with rasterio.open(fp, "r") as ds:
        cmap = ds.colormap(1)
        assert len(cmap) == 256
        assert cmap[0] == exp_first_color
        assert cmap[127] == exp_middle_color
        assert cmap[255] == exp_last_color



#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2022 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""The AMI L1B Reader operates on Level 1B (L1B) NetCDF files for the
Advanced Meteorological Imager (AMI) instrument on board the Korean
Meteorological Administration (KMA) GEO-KOMPSAT-2A (GK-2A)
satellite. Geo2Grid determines if it supports files
based on the input filename. Files for AMI supported by Geo2Grid usually
have the following naming scheme::

    gk2a_ami_le1b_vi004_fd010ge_201909251200.nc

The AMI L1B reader supports all instrument spectral bands, identified in
Geo2Grid as the products shown in the table below. The
AMI L1B reader can be provided to the main geo2grid.sh script
using the ``-r`` option and the reader name ``ami_l1b``.

The list of supported products includes true and natural color imagery.
These are created by means of a python based atmospheric Rayleigh
scattering correction algorithm that is executed as part of the |project| AMI
L1B reader, along with sharpening to the highest spatial resolution. For
more information on the creation of RGBs, please see the
:ref:`RGB section <getting_started_rgb>`.

+---------------------------+-----------------------------------------------------+
| **Product Name**          | **Description**                                     |
+===========================+=====================================================+
| VI004                     | 0.47um Reflectance Band                             |
+---------------------------+-----------------------------------------------------+
| VI005                     | 0.51um Reflectance Band                             |
+---------------------------+-----------------------------------------------------+
| VI006                     | 0.64um Reflectance Band                             |
+---------------------------+-----------------------------------------------------+
| VI008                     | 0.86um Reflectance Band                             |
+---------------------------+-----------------------------------------------------+
| NR013                     | 1.37um Reflectance Band                             |
+---------------------------+-----------------------------------------------------+
| NR016                     | 1.61um Reflectance Band                             |
+---------------------------+-----------------------------------------------------+
| SW038                     | 3.83um Brightness Temperature Band                  |
+---------------------------+-----------------------------------------------------+
| WV063                     | 6.21um Brightness Temperature Band                  |
+---------------------------+-----------------------------------------------------+
| WV069                     | 6.94um Brightness Temperature Band                  |
+---------------------------+-----------------------------------------------------+
| WV073                     | 7.33um Brightness Temperature Band                  |
+---------------------------+-----------------------------------------------------+
| IR087                     | 8.59um Brightness Temperature Band                  |
+---------------------------+-----------------------------------------------------+
| IR096                     | 9.62um Brightness Temperature Band                  |
+---------------------------+-----------------------------------------------------+
| IR105                     | 10.35um Brightness Temperature Band                 |
+---------------------------+-----------------------------------------------------+
| IR112                     | 11.23um Brightness Temperature Band                 |
+---------------------------+-----------------------------------------------------+
| IR123                     | 12.36um Brightness Temperature Band                 |
+---------------------------+-----------------------------------------------------+
| IR133                     | 13.29um Brightness Temperature Band                 |
+---------------------------+-----------------------------------------------------+
| true_color                | Ratio sharpened rayleigh corrected true color       |
+---------------------------+-----------------------------------------------------+
| natural_color             | Ratio sharpened corrected natural color             |
+---------------------------+-----------------------------------------------------+
| airmass                   | Air mass RGB                                        |
+---------------------------+-----------------------------------------------------+
| ash                       | Ash RGB                                             |
+---------------------------+-----------------------------------------------------+

"""

from __future__ import annotations

from argparse import ArgumentParser, _ArgumentGroup
from typing import Optional

from ._base import ReaderProxyBase

READER_PRODUCTS = [
    "VI004",
    "VI005",
    "VI006",
    "VI008",
    "NR013",
    "NR016",
    "SW038",
    "WV063",
    "WV069",
    "WV073",
    "IR087",
    "IR096",
    "IR105",
    "IR112",
    "IR123",
    "IR133",
]
COMPOSITE_PRODUCTS = [
    "true_color",
    "natural_color",
    "airmass",
    "ash",
    "dust",
    "fog",
    "night_microphysics",
]


class ReaderProxy(ReaderProxyBase):
    """Provide Polar2Grid-specific information about this reader's products."""

    is_geo2grid_reader = True

    def get_default_products(self) -> list[str]:
        """Get products to load if users hasn't specified any others."""
        return READER_PRODUCTS + COMPOSITE_PRODUCTS[:2]

    def get_all_products(self) -> list[str]:
        """Get all polar2grid products that could be loaded."""
        return READER_PRODUCTS + COMPOSITE_PRODUCTS


def add_reader_argument_groups(
    parser: ArgumentParser, group: Optional[_ArgumentGroup] = None
) -> tuple[Optional[_ArgumentGroup], Optional[_ArgumentGroup]]:
    """Add reader-specific command line arguments to an existing argument parser.

    If ``group`` is provided then arguments are added to this group. If not,
    a new group is added to the parser and arguments added to this new group.

    """
    if group is None:
        group = parser.add_argument_group(title="ABI L1b Reader")
    return group, None



#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2021 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Tests for the hdf5 writer."""

import os

import pytest
import satpy

TEST_ETC_DIR = os.path.realpath(os.path.join(os.path.dirname(__file__), "..", "etc"))


class TestHDF5Writer:
    def setup_method(self):
        """Add P2G configs to the Satpy path."""
        from polar2grid.utils.config import add_polar2grid_config_paths

        self._old_path = satpy.config.get("config_path")
        add_polar2grid_config_paths()
        # add test specific configs
        curr_path = satpy.config.get("config_path")
        satpy.config.set(config_path=[TEST_ETC_DIR] + curr_path)

    def teardown_method(self):
        """Reset Satpy config path back to the original value."""
        satpy.config.set(config_path=self._old_path)

    @pytest.mark.parametrize("add_geolocation", [False, True])
    def test_hdf5_basic(self, add_geolocation, abi_l1b_c01_scene, tmp_path):
        """Test basic writing of gridded data."""
        import h5py

        abi_l1b_c01_scene.load(["C01"])
        abi_l1b_c01_scene.save_datasets(
            writer="hdf5",
            base_dir=str(tmp_path),
            filename="{platform_name}_{sensor}_{start_time:%Y%m%d_%H%M%S}.h5",
            add_geolocation=add_geolocation,
        )
        exp_fn = tmp_path / "goes16_abi_20210101_120000.h5"

        assert os.path.isfile(exp_fn)
        hdf_file = h5py.File(exp_fn, "r")
        assert "goes_east" in hdf_file
        assert "C01" in hdf_file["goes_east"]
        if add_geolocation:
            assert "longitude" in hdf_file["goes_east"]
            assert "latitude" in hdf_file["goes_east"]
        else:
            assert "longitude" not in hdf_file["goes_east"]
            assert "latitude" not in hdf_file["goes_east"]
